# cython: always_allow_keywords=True
# mode: run
# tag: posonly, pure3.8

import cython
import pickle

def test_optional_posonly_args1(a, b=10, /, c=100):
    """
    >>> test_optional_posonly_args1(1, 2, 3)
    6
    >>> test_optional_posonly_args1(1, 2, c=3)
    6
    >>> test_optional_posonly_args1(1, b=2, c=3)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_optional_posonly_args1() got ... keyword argument... 'b'
    >>> test_optional_posonly_args1(1, 2)
    103
    >>> test_optional_posonly_args1(1, b=2)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_optional_posonly_args1() got ... keyword argument... 'b'
    """
    return a + b + c

def test_optional_posonly_args2(a=1, b=10, /, c=100):
    """
    >>> test_optional_posonly_args2(1, 2, 3)
    6
    >>> test_optional_posonly_args2(1, 2, c=3)
    6
    >>> test_optional_posonly_args2(1, b=2, c=3)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_optional_posonly_args2() got ... keyword argument... 'b'
    >>> test_optional_posonly_args2(1, 2)
    103
    >>> test_optional_posonly_args2(1, b=2)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_optional_posonly_args2() got ... keyword argument... 'b'
    >>> test_optional_posonly_args2(1, c=2)
    13
    """
    return a + b + c

# TODO: this causes a line that is too long for old versions of Clang
#def many_args(a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16,a17,a18,a19,a20,a21,
#              a22,a23,a24,a25,a26,a27,a28,a29,a30,a31,a32,a33,a34,a35,a36,a37,a38,a39,a40,
#              a41,a42,a43,a44,a45,a46,a47,a48,a49,a50,a51,a52,a53,a54,a55,a56,a57,a58,a59,
#              a60,a61,a62,a63,a64,a65,a66,a67,a68,a69,a70,a71,a72,a73,a74,a75,a76,a77,a78,
#              a79,a80,a81,a82,a83,a84,a85,a86,a87,a88,a89,a90,a91,a92,a93,a94,a95,a96,a97,
#              a98,a99,a100,a101,a102,a103,a104,a105,a106,a107,a108,a109,a110,a111,a112,
#              a113,a114,a115,a116,a117,a118,a119,a120,a121,a122,a123,a124,a125,a126,a127,
#              a128,a129,a130,a131,a132,a133,a134,a135,a136,a137,a138,a139,a140,a141,a142,
#              a143,a144,a145,a146,a147,a148,a149,a150,a151,a152,a153,a154,a155,a156,a157,
#              a158,a159,a160,a161,a162,a163,a164,a165,a166,a167,a168,a169,a170,a171,a172,
#              a173,a174,a175,a176,a177,a178,a179,a180,a181,a182,a183,a184,a185,a186,a187,
#              a188,a189,a190,a191,a192,a193,a194,a195,a196,a197,a198,a199,a200,a201,a202,
#              a203,a204,a205,a206,a207,a208,a209,a210,a211,a212,a213,a214,a215,a216,a217,
#              a218,a219,a220,a221,a222,a223,a224,a225,a226,a227,a228,a229,a230,a231,a232,
#              a233,a234,a235,a236,a237,a238,a239,a240,a241,a242,a243,a244,a245,a246,a247,
#              a248,a249,a250,a251,a252,a253,a254,a255,a256,a257,a258,a259,a260,a261,a262,
#              a263,a264,a265,a266,a267,a268,a269,a270,a271,a272,a273,a274,a275,a276,a277,
#              a278,a279,a280,a281,a282,a283,a284,a285,a286,a287,a288,a289,a290,a291,a292,
#              a293,a294,a295,a296,a297,a298,a299,/,b,c=42,*,d):
#    """
#    >>> many_args(*range(299),b=1,c=2,d=3)
#    (298, 1, 2, 3)
#    >>> many_args(*range(299),b=1,d=3)
#    (298, 1, 42, 3)
#    >>> many_args(*range(300),d=3)
#    (298, 299, 42, 3)
#    """
#    return (a299, b, c, d)

#TODO: update this test for Python 3.8 final
@cython.binding(True)
def func_introspection1(a, b, c, /, d, e=1, *, f, g=2):
    """
    >>> assert func_introspection2.__code__.co_argcount == 5, func_introspection2.__code__.co_argcount
    >>> func_introspection1.__defaults__
    (1,)
    """

@cython.binding(True)
def func_introspection2(a, b, c=1, /, d=2, e=3, *, f, g=4):
    """
    >>> assert func_introspection2.__code__.co_argcount == 5, func_introspection2.__code__.co_argcount
    >>> func_introspection2.__defaults__
    (1, 2, 3)
    """

def test_pos_only_call_via_unpacking(a, b, /):
    """
    >>> test_pos_only_call_via_unpacking(*[1,2])
    3
    """
    return a + b

def test_use_positional_as_keyword1(a, /):
    """
    >>> test_use_positional_as_keyword1(1)
    >>> test_use_positional_as_keyword1(a=1)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_use_positional_as_keyword1() ... keyword argument...
    """

def test_use_positional_as_keyword2(a, /, b):
    """
    >>> test_use_positional_as_keyword2(1, 2)
    >>> test_use_positional_as_keyword2(1, b=2)
    >>> test_use_positional_as_keyword2(a=1, b=2)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_use_positional_as_keyword2() ... positional...argument...
    """

def test_use_positional_as_keyword3(a, b, /):
    """
    >>> test_use_positional_as_keyword3(1, 2)
    >>> test_use_positional_as_keyword3(a=1, b=2) # doctest:+ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_use_positional_as_keyword3() got ... keyword argument...
    """

def test_positional_only_and_arg_invalid_calls(a, b, /, c):
    """
    >>> test_positional_only_and_arg_invalid_calls(1, 2, 3)
    >>> test_positional_only_and_arg_invalid_calls(1, 2, c=3)
    >>> test_positional_only_and_arg_invalid_calls(1, 2)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_and_arg_invalid_calls() ... positional argument...
    >>> test_positional_only_and_arg_invalid_calls(1)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_and_arg_invalid_calls() ... positional arguments...
    >>> test_positional_only_and_arg_invalid_calls(1,2,3,4)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_and_arg_invalid_calls() takes ... positional arguments ...4 ...given...
    """

def test_positional_only_and_optional_arg_invalid_calls(a, b, /, c=3):
    """
    >>> test_positional_only_and_optional_arg_invalid_calls(1, 2)
    >>> test_positional_only_and_optional_arg_invalid_calls(1)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_and_optional_arg_invalid_calls() ... positional argument...
    >>> test_positional_only_and_optional_arg_invalid_calls()  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_and_optional_arg_invalid_calls() ... positional arguments...
    >>> test_positional_only_and_optional_arg_invalid_calls(1, 2, 3, 4)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_and_optional_arg_invalid_calls() takes ... positional arguments ...4 ...given...
    """

def test_positional_only_and_kwonlyargs_invalid_calls(a, b, /, c, *, d, e):
    """
    >>> test_positional_only_and_kwonlyargs_invalid_calls(1, 2, 3, d=1, e=2)
    >>> test_positional_only_and_kwonlyargs_invalid_calls(1, 2, 3, e=2)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_and_kwonlyargs_invalid_calls() ... keyword-only argument...d...
    >>> test_positional_only_and_kwonlyargs_invalid_calls(1, 2, 3)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_and_kwonlyargs_invalid_calls() ... keyword-only argument...d...
    >>> test_positional_only_and_kwonlyargs_invalid_calls(1, 2)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_and_kwonlyargs_invalid_calls() ... positional argument...
    >>> test_positional_only_and_kwonlyargs_invalid_calls(1)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_and_kwonlyargs_invalid_calls() ... positional arguments...
    >>> test_positional_only_and_kwonlyargs_invalid_calls()  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_and_kwonlyargs_invalid_calls() ... positional arguments...
    >>> test_positional_only_and_kwonlyargs_invalid_calls(1, 2, 3, 4, 5, 6, d=7, e=8)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_and_kwonlyargs_invalid_calls() takes ... positional arguments ...
    >>> test_positional_only_and_kwonlyargs_invalid_calls(1, 2, 3, d=1, e=4, f=56)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_and_kwonlyargs_invalid_calls() got an unexpected keyword argument 'f'
    """

def test_positional_only_invalid_calls(a, b, /):
    """
    >>> test_positional_only_invalid_calls(1, 2)
    >>> test_positional_only_invalid_calls(1)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_invalid_calls() ... positional argument...
    >>> test_positional_only_invalid_calls()  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_invalid_calls() ... positional arguments...
    >>> test_positional_only_invalid_calls(1, 2, 3)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_invalid_calls() takes ... positional arguments ...3 ...given...
    """

def test_positional_only_with_optional_invalid_calls(a, b=2, /):
    """
    >>> test_positional_only_with_optional_invalid_calls(1)
    >>> test_positional_only_with_optional_invalid_calls()  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_with_optional_invalid_calls() ... positional argument...
    >>> test_positional_only_with_optional_invalid_calls(1, 2, 3)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_positional_only_with_optional_invalid_calls() takes ... positional arguments ...3 ...given...
    """

def test_no_standard_args_usage(a, b, /, *, c):
    """
    >>> test_no_standard_args_usage(1, 2, c=3)
    >>> test_no_standard_args_usage(1, b=2, c=3)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_no_standard_args_usage() ... positional... argument...
    """

#def test_change_default_pos_only():
# TODO: probably remove this, since  __defaults__ is not writable in Cython?
#    """
#    >>> test_change_default_pos_only()
#    True
#    True
#    """
#    def f(a, b=2, /, c=3):
#        return a + b + c
#
#    print((2,3) == f.__defaults__)
#    f.__defaults__ = (1, 2, 3)
#    print(f(1, 2, 3) == 6)

def test_lambdas():
    """
    >>> test_lambdas()
    3
    3
    3
    3
    3
    """
    x = lambda a, /, b: a + b
    print(x(1,2))
    print(x(1,b=2))

    x = lambda a, /, b=2: a + b
    print(x(1))

    x = lambda a, b, /: a + b
    print(x(1, 2))

    x = lambda a, b, /, : a + b
    print(x(1, 2))

class TestPosonlyMethods(object):
    """
    >>> TestPosonlyMethods().f(1,2)
    (1, 2)
    >>> TestPosonlyMethods.f(TestPosonlyMethods(), 1, 2)
    (1, 2)
    >>> try:
    ...     TestPosonlyMethods.f(1,2)
    ... except TypeError:
    ...    print("Got type error")
    Got type error
    >>> TestPosonlyMethods().f(1, b=2)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...f() got ... keyword argument... 'b'
    """
    def f(self, a, b, /):
        return a, b

class TestMangling(object):
    """
    >>> TestMangling().f()
    42
    >>> TestMangling().f2()
    42

    #>>> TestMangling().f3()
    #(42, 43)
    #>>> TestMangling().f4()
    #(42, 43, 44)

    >>> TestMangling().f2(1)
    1

    #>>> TestMangling().f3(1, _TestMangling__b=2)
    #(1, 2)
    #>>> TestMangling().f4(1, _TestMangling__b=2, _TestMangling__c=3)
    #(1, 2, 3)
    """
    def f(self, *, __a=42):
        return __a

    def f2(self, __a=42, /):
        return __a

# FIXME: https://github.com/cython/cython/issues/1382
#    def f3(self, __a=42, /, __b=43):
#        return (__a, __b)

#    def f4(self, __a=42, /, __b=43, *, __c=44):
#        return (__a, __b, __c)

def test_module_function(a, b, /):
    """
    >>> test_module_function(1, 2)
    >>> test_module_function()  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_module_function() ... positional arguments...
    """

def test_closures1(x,y):
    """
    >>> test_closures1(1,2)(3,4)
    10
    >>> test_closures1(1,2)(3)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...g() ... positional argument...
    >>> test_closures1(1,2)(3,4,5)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...g() ... positional argument...
    """
    def g(x2, /, y2):
        return x + y + x2 + y2
    return g

def test_closures2(x, /, y):
    """
    >>> test_closures2(1,2)(3,4)
    10
    """
    def g(x2,y2):
        return x + y + x2 + y2
    return g


def test_closures3(x, /, y):
    """
    >>> test_closures3(1,2)(3,4)
    10
    >>> test_closures3(1,2)(3)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...g() ... positional argument...
    >>> test_closures3(1,2)(3,4,5)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...g() ... positional argument...
    """
    def g(x2, /, y2):
        return x + y + x2 + y2
    return g


def test_same_keyword_as_positional_with_kwargs(something, /, **kwargs):
    """
    >>> test_same_keyword_as_positional_with_kwargs(42, something=42)
    (42, {'something': 42})
    >>> test_same_keyword_as_positional_with_kwargs(something=42)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_same_keyword_as_positional_with_kwargs() ... positional argument...
    >>> test_same_keyword_as_positional_with_kwargs(42)
    (42, {})
    """
    return (something, kwargs)

def test_serialization1(a, b, /):
    """
    >>> pickled_posonly = pickle.dumps(test_serialization1)
    >>> unpickled_posonly = pickle.loads(pickled_posonly)
    >>> unpickled_posonly(1, 2)
    (1, 2)
    >>> unpickled_posonly(a=1, b=2)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_serialization1() got ... keyword argument...
    """
    return (a, b)

def test_serialization2(a, /, b):
    """
    >>> pickled_optional = pickle.dumps(test_serialization2)
    >>> unpickled_optional = pickle.loads(pickled_optional)
    >>> unpickled_optional(1, 2)
    (1, 2)
    >>> unpickled_optional(a=1, b=2)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_serialization2() ... positional... argument...
    """
    return (a, b)

def test_serialization3(a=1, /, b=2):
    """
    >>> pickled_defaults = pickle.dumps(test_serialization3)
    >>> unpickled_defaults = pickle.loads(pickled_defaults)
    >>> unpickled_defaults(1, 2)
    (1, 2)
    >>> unpickled_defaults(a=1, b=2)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_serialization3() got ... keyword argument... 'a'
    """
    return (a, b)


async def test_async(a=1, /, b=2):
    """
    >>> test_async(a=1, b=2)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_async() got ... keyword argument... 'a'
    """
    return a, b


def test_async_call(*args, **kwargs):
    """
    >>> test_async_call(1, 2)
    >>> test_async_call(1, b=2)
    >>> test_async_call(1)
    >>> test_async_call()
    """
    try:
        coro = test_async(*args, **kwargs)
        coro.send(None)
    except StopIteration as e:
        result = e.value
    assert result == (1, 2), result


def test_generator(a=1, /, b=2):
    """
    >>> test_generator(a=1, b=2)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: test_generator() got ... keyword argument... 'a'
    >>> gen = test_generator(1, 2)
    >>> next(gen)
    (1, 2)
    >>> gen = test_generator(1, b=2)
    >>> next(gen)
    (1, 2)
    >>> gen = test_generator(1)
    >>> next(gen)
    (1, 2)
    >>> gen = test_generator()
    >>> next(gen)
    (1, 2)
    """
    yield a, b

def f_call_1_0_0(a,/):
    """
    >>> f_call_1_0_0(1)
    (1,)
    """
    return (a,)

def f_call_1_1_0(a, /, b):
    """
    >>> f_call_1_1_0(1,2)
    (1, 2)
    """
    return (a,b)

def f_call_1_1_1(a, /, b, *, c):
    """
    >>> f_call_1_1_1(1,2,c=3)
    (1, 2, 3)
    """
    return (a,b,c)

def f_call_1_1_1_star(a, /, b, *args, c):
    """
    >>> f_call_1_1_1_star(1,2,c=3)
    (1, 2, (), 3)
    >>> f_call_1_1_1_star(1,2,3,4,5,6,7,8,c=9)
    (1, 2, (3, 4, 5, 6, 7, 8), 9)
    """
    return (a,b,args,c)

def f_call_1_1_1_kwds(a, /, b, *, c, **kwds):
    """
    >>> f_call_1_1_1_kwds(1,2,c=3)
    (1, 2, 3, {})
    >>> f_call_1_1_1_kwds(1,2,c=3,d=4,e=5) == (1, 2, 3, {'d': 4, 'e': 5})
    True
    """
    return (a,b,c,kwds)

def f_call_1_1_1_star_kwds(a, /, b, *args, c, **kwds):
    """
    >>> f_call_1_1_1_star_kwds(1,2,c=3,d=4,e=5) == (1, 2, (), 3, {'d': 4, 'e': 5})
    True
    >>> f_call_1_1_1_star_kwds(1,2,3,4,c=5,d=6,e=7) == (1, 2, (3, 4), 5, {'d': 6, 'e': 7})
    True
    """
    return (a,b,args,c,kwds)

def f_call_one_optional_kwd(a, /, *, b=2):
    """
    >>> f_call_one_optional_kwd(1)
    (1, 2)
    >>> f_call_one_optional_kwd(1, b=3)
    (1, 3)
    """
    return (a,b)

def f_call_posonly_stararg(a, /, *args):
    """
    >>> f_call_posonly_stararg(1)
    (1, ())
    >>> f_call_posonly_stararg(1, 2, 3, 4)
    (1, (2, 3, 4))
    """
    return (a,args)

def f_call_posonly_kwarg(a, /, **kw):
    """
    >>> f_call_posonly_kwarg(1)
    (1, {})
    >>> all_args = f_call_posonly_kwarg(1, b=2, c=3, d=4)
    >>> all_args == (1, {'b': 2, 'c': 3, 'd': 4}) or all_args
    True
    """
    return (a,kw)

def f_call_posonly_stararg_kwarg(a, /, *args, **kw):
    """
    >>> f_call_posonly_stararg_kwarg(1)
    (1, (), {})
    >>> f_call_posonly_stararg_kwarg(1, 2)
    (1, (2,), {})
    >>> all_args = f_call_posonly_stararg_kwarg(1, b=3, c=4)
    >>> all_args == (1, (), {'b': 3, 'c': 4}) or all_args
    True
    >>> all_args = f_call_posonly_stararg_kwarg(1, 2, b=3, c=4)
    >>> all_args == (1, (2,), {'b': 3, 'c': 4}) or all_args
    True
    """
    return (a,args,kw)

def test_empty_kwargs(a, b, /):
    """
    >>> test_empty_kwargs(1, 2)
    (1, 2)
    >>> test_empty_kwargs(1, 2, **{})
    (1, 2)
    >>> test_empty_kwargs(1, 2, **{'c': 3})
    Traceback (most recent call last):
    TypeError: test_empty_kwargs() got an unexpected keyword argument 'c'
    """
    return (a,b)


@cython.cclass
class TestExtensionClass:
    """
    >>> t = TestExtensionClass()
    >>> t.f(1,2)
    (1, 2, 3)
    >>> t.f(1,2,4)
    (1, 2, 4)
    >>> t.f(1, 2, c=4)
    (1, 2, 4)
    >>> t.f(1, 2, 5, c=6)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...f() got multiple values for ...argument 'c'
    """
    def f(self, a, b, /, c=3):
        return (a,b,c)



# mode: error

cdef class A:
    cdef int value = 3

cdef extern from *:
    cdef struct B:
        int value = 3

_ERRORS = u"""
4:13: Cannot assign default value to fields in cdef classes, structs or unions
8:12: Cannot assign default value to fields in cdef classes, structs or unions
"""



# mode: compile

cdef:

    struct PrivFoo:
        int i

    int priv_i

    void priv_f():
        global priv_i
        priv_i = 42

cdef public:

    struct PubFoo:
        int i

    int pub_v

    void pub_f():
        pass

    class PubBlarg [object PubBlargObj, type PubBlargType]:
        pass

cdef api:

    void api_f():
        pass

cdef public api:

    void pub_api_f():
        pass

priv_f()



cdef extern from "complexobject.h":

    struct Py_complex:
        double real
        double imag

    ctypedef class __builtin__.complex [object PyComplexObject]:
        cdef Py_complex cval

        @property
        cdef inline double real(self):
            return self.cval.real

        @property
        cdef inline double imag(self):
            return self.cval.imag


def cprint(complex c):
    print(f"{c.real :.4f}{c.imag :+.4f}j")  # uses C calls to the above property methods.



# mode: run
# tag: autowrap
# cython: always_allow_keywords=True

cdef void memoryview_func_a(double [:] x):
    x[0] = 1

cdef void memoryview_func_b(double [::1] x):
    x[0] = 2

cdef void memoryview_func_c(int [:] x):
    x[0] = 1

cdef void memoryview_func_d(int [:] x):
    x[0] = 2

cdef void memoryview_func_e(int [:,::1] x):
    x[0,0] = 4

cdef void memoryview_func_f(int [::1,:] x):
    x[0,0] = 4


def test_memview_wrapping():
    """
    We're mainly concerned that the code compiles without the names clashing
    >>> test_memview_wrapping()
    1.0
    2.0
    1
    2
    """
    cdef a = memoryview_func_a
    cdef b = memoryview_func_b
    cdef c = memoryview_func_c
    cdef d = memoryview_func_d
    cdef e = memoryview_func_e
    cdef f = memoryview_func_f
    cdef double[1] double_arr = [0]
    cdef int[1] int_arr = [0]

    a(<double[:1]> double_arr)
    print(double_arr[0])
    b(<double[:1:1]> double_arr)
    print(double_arr[0])
    c(<int[:1]> int_arr)
    print(int_arr[0])
    d(<int[:1:1]> int_arr)
    print(int_arr[0])
    # don't call e and f because it's harder without needing extra dependencies
    # it's mostly a compile test for them



# ticket: t87

__doc__ = u"""
>>> d = Defined()
>>> n = NotDefined()         # doctest: +ELLIPSIS
Traceback (most recent call last):
NameError: ...name 'NotDefined' is not defined
"""

if True:
    class Defined(object):
        """
        >>> isinstance(Defined(), Defined)
        True
        """

if False:
    class NotDefined(object):
        """
        >>> NotDefined() # fails when defined
        """

def test_class_cond(x):
    """
    >>> Test, test = test_class_cond(True)
    >>> test.A
    1
    >>> Test().A
    1
    >>> Test, test = test_class_cond(False)
    >>> test.A
    2
    >>> Test().A
    2
    """
    if x:
        class Test(object):
            A = 1
    else:
        class Test(object):
            A = 2
    return Test, Test()

def test_func_cond(x):
    """
    >>> func = test_func_cond(True)
    >>> func()
    1
    >>> func = test_func_cond(False)
    >>> func()
    2
    """
    if x:
        def func():
            return 1
    else:
        def func():
            return 2
    return func



# Verify defined before function prototype
cdef (int, int) get_a_ctuple()

# Verify defined before typedef
ctypedef (int, double) int_double

# Verify typedef defined
cdef int_double tuple_global = (1, 2.)

# Verify defined before opt args
cdef void test_opt_args((double, int) x=*)

# Verify defined before class declaration
cdef class CTupleClass:
    cdef void get_a_ctuple(self, (double, double) x)



import unittest
from io import StringIO
import string

from .. import Scanning
from ..Symtab import ModuleScope
from ..TreeFragment import StringParseContext
from ..Errors import init_thread

# generate some fake code - just a bunch of lines of the form "a0 a1 ..."
code = []
for ch in string.ascii_lowercase:
    line = " ".join(["%s%s" % (ch, n) for n in range(10)])
    code.append(line)
code = "\n".join(code)

init_thread()


class TestScanning(unittest.TestCase):
    def make_scanner(self):
        source = Scanning.StringSourceDescriptor("fake code", code)
        buf = StringIO(code)
        context = StringParseContext("fake context")
        scope = ModuleScope("fake_module", None, None)

        return Scanning.PyrexScanner(buf, source, scope=scope, context=context)

    def test_put_back_positions(self):
        scanner = self.make_scanner()

        self.assertEqual(scanner.sy, "IDENT")
        self.assertEqual(scanner.systring, "a0")
        scanner.next()
        self.assertEqual(scanner.sy, "IDENT")
        self.assertEqual(scanner.systring, "a1")
        a1pos = scanner.last_token_position_tuple
        self.assertEqual(a1pos[1:], (1, 3))
        a2peek = scanner.peek()  # shouldn't mess up the position
        self.assertEqual(a1pos, scanner.last_token_position_tuple)
        scanner.next()
        self.assertEqual(a2peek, (scanner.sy, scanner.systring))

        # find next line
        while scanner.sy != "NEWLINE":
            scanner.next()

        line_sy = []
        line_systring = []
        line_pos = []

        scanner.next()
        while scanner.sy != "NEWLINE":
            line_sy.append(scanner.sy)
            line_systring.append(scanner.systring)
            line_pos.append(scanner.last_token_position_tuple)
            scanner.next()

        for sy, systring, pos in zip(
            line_sy[::-1], line_systring[::-1], line_pos[::-1]
        ):
            scanner.put_back(sy, systring, pos)

        n = 0
        while scanner.sy != "NEWLINE":
            self.assertEqual(scanner.sy, line_sy[n])
            self.assertEqual(scanner.systring, line_systring[n])
            self.assertEqual(scanner.last_token_position_tuple, line_pos[n])
            scanner.next()
            n += 1

        self.assertEqual(n, len(line_pos))

    def test_tentatively_scan(self):
        scanner = self.make_scanner()
        with Scanning.tentatively_scan(scanner) as errors:
            while scanner.sy != "NEWLINE":
                scanner.next()
        self.assertFalse(errors)

        scanner.next()
        self.assertEqual(scanner.systring, "b0")
        pos = scanner.last_token_position_tuple
        with Scanning.tentatively_scan(scanner) as errors:
            while scanner.sy != "NEWLINE":
                scanner.next()
                if scanner.systring == "b7":
                    scanner.error("Oh no not b7!")
                    break
        self.assertTrue(errors)
        self.assertEqual(scanner.systring, "b0")  # state has been restored
        self.assertEqual(scanner.last_token_position_tuple, pos)
        scanner.next()
        self.assertEqual(scanner.systring, "b1")  # and we can keep going again
        scanner.next()
        self.assertEqual(scanner.systring, "b2")  # and we can keep going again

        with Scanning.tentatively_scan(scanner) as error:
            scanner.error("Something has gone wrong with the current symbol")
        self.assertEqual(scanner.systring, "b2")
        scanner.next()
        self.assertEqual(scanner.systring, "b3")

        # test a few combinations of nested scanning
        sy1, systring1 = scanner.sy, scanner.systring
        pos1 = scanner.last_token_position_tuple
        with Scanning.tentatively_scan(scanner):
            scanner.next()
            sy2, systring2 = scanner.sy, scanner.systring
            pos2 = scanner.last_token_position_tuple
            with Scanning.tentatively_scan(scanner):
                with Scanning.tentatively_scan(scanner):
                    scanner.next()
                    scanner.next()
                    scanner.error("Ooops")
                self.assertEqual((scanner.sy, scanner.systring), (sy2, systring2))
            self.assertEqual((scanner.sy, scanner.systring), (sy2, systring2))
            scanner.error("eee")
        self.assertEqual((scanner.sy, scanner.systring), (sy1, systring1))
        with Scanning.tentatively_scan(scanner):
            scanner.next()
            scanner.next()
            with Scanning.tentatively_scan(scanner):
                scanner.next()
                # no error - but this block should be unwound by the outer block too
            scanner.next()
            scanner.error("Oooops")
        self.assertEqual((scanner.sy, scanner.systring), (sy1, systring1))




if __name__ == "__main__":
    unittest.main()



#
# Nodes used as utilities and support for transforms etc.
# These often make up sets including both Nodes and ExprNodes
# so it is convenient to have them in a separate module.
#


from . import Nodes
from . import ExprNodes
from .Nodes import Node
from .ExprNodes import AtomicExprNode
from .PyrexTypes import c_ptr_type, c_int_type


class TempHandle:
    # THIS IS DEPRECATED, USE LetRefNode instead
    temp = None
    needs_xdecref = False
    def __init__(self, type, needs_cleanup=None):
        self.type = type
        if needs_cleanup is None:
            self.needs_cleanup = type.is_pyobject
        else:
            self.needs_cleanup = needs_cleanup

    def ref(self, pos):
        return TempRefNode(pos, handle=self, type=self.type)


class TempRefNode(AtomicExprNode):
    # THIS IS DEPRECATED, USE LetRefNode instead
    # handle   TempHandle

    def analyse_types(self, env):
        assert self.type == self.handle.type
        return self

    def analyse_target_types(self, env):
        assert self.type == self.handle.type
        return self

    def analyse_target_declaration(self, env):
        pass

    def calculate_result_code(self):
        result = self.handle.temp
        if result is None: result = "<error>"  # might be called and overwritten
        return result

    def generate_result_code(self, code):
        pass

    def generate_assignment_code(self, rhs, code, overloaded_assignment=False):
        if self.type.is_pyobject:
            rhs.make_owned_reference(code)
            # TODO: analyse control flow to see if this is necessary
            code.put_xdecref(self.result(), self.ctype())
        code.putln('%s = %s;' % (
            self.result(),
            rhs.result() if overloaded_assignment else rhs.result_as(self.ctype()),
        ))
        rhs.generate_post_assignment_code(code)
        rhs.free_temps(code)


class TempsBlockNode(Node):
    # THIS IS DEPRECATED, USE LetNode instead

    """
    Creates a block which allocates temporary variables.
    This is used by transforms to output constructs that need
    to make use of a temporary variable. Simply pass the types
    of the needed temporaries to the constructor.

    The variables can be referred to using a TempRefNode
    (which can be constructed by calling get_ref_node).
    """

    # temps   [TempHandle]
    # body    StatNode

    child_attrs = ["body"]

    def generate_execution_code(self, code):
        for handle in self.temps:
            handle.temp = code.funcstate.allocate_temp(
                handle.type, manage_ref=handle.needs_cleanup)
        self.body.generate_execution_code(code)
        for handle in self.temps:
            if handle.needs_cleanup:
                if handle.needs_xdecref:
                    code.put_xdecref_clear(handle.temp, handle.type)
                else:
                    code.put_decref_clear(handle.temp, handle.type)
            code.funcstate.release_temp(handle.temp)

    def analyse_declarations(self, env):
        self.body.analyse_declarations(env)

    def analyse_expressions(self, env):
        self.body = self.body.analyse_expressions(env)
        return self

    def generate_function_definitions(self, env, code):
        self.body.generate_function_definitions(env, code)

    def annotate(self, code):
        self.body.annotate(code)


class ResultRefNode(AtomicExprNode):
    # A reference to the result of an expression.  The result_code
    # must be set externally (usually a temp name).

    subexprs = []
    lhs_of_first_assignment = False

    def __init__(self, expression=None, pos=None, type=None, may_hold_none=True, is_temp=False):
        self.expression = expression
        self.pos = None
        self.may_hold_none = may_hold_none
        if expression is not None:
            self.pos = expression.pos
            self.type = getattr(expression, "type", None)
        if pos is not None:
            self.pos = pos
        if type is not None:
            self.type = type
        if is_temp:
            self.is_temp = True
        assert self.pos is not None

    def clone_node(self):
        # nothing to do here
        return self

    def type_dependencies(self, env):
        if self.expression:
            return self.expression.type_dependencies(env)
        else:
            return ()

    def update_expression(self, expression):
        self.expression = expression
        type = getattr(expression, "type", None)
        if type:
            self.type = type

    def analyse_target_declaration(self, env):
        pass  # OK - we can assign to this

    def analyse_types(self, env):
        if self.expression is not None:
            if not self.expression.type:
                self.expression = self.expression.analyse_types(env)
            self.type = self.expression.type
        return self

    def infer_type(self, env):
        if self.type is not None:
            return self.type
        if self.expression is not None:
            if self.expression.type is not None:
                return self.expression.type
            return self.expression.infer_type(env)
        assert False, "cannot infer type of ResultRefNode"

    def may_be_none(self):
        if not self.type.is_pyobject:
            return False
        return self.may_hold_none

    def _DISABLED_may_be_none(self):
        # not sure if this is safe - the expression may not be the
        # only value that gets assigned
        if self.expression is not None:
            return self.expression.may_be_none()
        if self.type is not None:
            return self.type.is_pyobject
        return True  # play it safe

    def is_simple(self):
        return True

    def result(self):
        try:
            return self.result_code
        except AttributeError:
            if self.expression is not None:
                self.result_code = self.expression.result()
        return self.result_code

    def generate_evaluation_code(self, code):
        pass

    def generate_result_code(self, code):
        pass

    def generate_disposal_code(self, code):
        pass

    def generate_assignment_code(self, rhs, code, overloaded_assignment=False):
        if self.type.is_pyobject:
            rhs.make_owned_reference(code)
            if not self.lhs_of_first_assignment:
                code.put_decref(self.result(), self.ctype())
        code.putln('%s = %s;' % (
            self.result(),
            rhs.result() if overloaded_assignment else rhs.result_as(self.ctype()),
        ))
        rhs.generate_post_assignment_code(code)
        rhs.free_temps(code)

    def allocate_temps(self, env):
        pass

    def release_temp(self, env):
        pass

    def free_temps(self, code):
        pass


class LetNodeMixin:
    def set_temp_expr(self, lazy_temp):
        self.lazy_temp = lazy_temp
        self.temp_expression = lazy_temp.expression

    def setup_temp_expr(self, code):
        self.temp_expression.generate_evaluation_code(code)
        self.temp_type = self.temp_expression.type
        if self.temp_type.is_array:
            self.temp_type = c_ptr_type(self.temp_type.base_type)
        self._result_in_temp = self.temp_expression.result_in_temp()
        if self._result_in_temp:
            self.temp = self.temp_expression.result()
        else:
            if self.temp_type.is_memoryviewslice:
                self.temp_expression.make_owned_memoryviewslice(code)
            else:
                self.temp_expression.make_owned_reference(code)
            self.temp = code.funcstate.allocate_temp(
                self.temp_type, manage_ref=True)
            code.putln("%s = %s;" % (self.temp, self.temp_expression.result()))
            self.temp_expression.generate_disposal_code(code)
            self.temp_expression.free_temps(code)
        self.lazy_temp.result_code = self.temp

    def teardown_temp_expr(self, code):
        if self._result_in_temp:
            self.temp_expression.generate_disposal_code(code)
            self.temp_expression.free_temps(code)
        else:
            if self.temp_type.needs_refcounting:
                code.put_decref_clear(self.temp, self.temp_type)
            code.funcstate.release_temp(self.temp)


class EvalWithTempExprNode(ExprNodes.ExprNode, LetNodeMixin):
    # A wrapper around a subexpression that moves an expression into a
    # temp variable and provides it to the subexpression.

    subexprs = ['temp_expression', 'subexpression']

    def __init__(self, lazy_temp, subexpression):
        self.set_temp_expr(lazy_temp)
        self.pos = subexpression.pos
        self.subexpression = subexpression
        # if called after type analysis, we already know the type here
        self.type = self.subexpression.type

    def infer_type(self, env):
        return self.subexpression.infer_type(env)

    def may_be_none(self):
        return self.subexpression.may_be_none()

    def result(self):
        return self.subexpression.result()

    def analyse_types(self, env):
        self.temp_expression = self.temp_expression.analyse_types(env)
        self.lazy_temp.update_expression(self.temp_expression)  # overwrite in case it changed
        self.subexpression = self.subexpression.analyse_types(env)
        self.type = self.subexpression.type
        return self

    def free_subexpr_temps(self, code):
        self.subexpression.free_temps(code)

    def generate_subexpr_disposal_code(self, code):
        self.subexpression.generate_disposal_code(code)

    def generate_evaluation_code(self, code):
        self.setup_temp_expr(code)
        self.subexpression.generate_evaluation_code(code)
        self.teardown_temp_expr(code)


LetRefNode = ResultRefNode


class LetNode(Nodes.StatNode, LetNodeMixin):
    # Implements a local temporary variable scope. Imagine this
    # syntax being present:
    # let temp = VALUE:
    #     BLOCK (can modify temp)
    #     if temp is an object, decref
    #
    # Usually used after analysis phase, but forwards analysis methods
    # to its children

    child_attrs = ['temp_expression', 'body']

    def __init__(self, lazy_temp, body):
        self.set_temp_expr(lazy_temp)
        self.pos = body.pos
        self.body = body

    def analyse_declarations(self, env):
        self.temp_expression.analyse_declarations(env)
        self.body.analyse_declarations(env)

    def analyse_expressions(self, env):
        self.temp_expression = self.temp_expression.analyse_expressions(env)
        self.body = self.body.analyse_expressions(env)
        return self

    def generate_execution_code(self, code):
        self.setup_temp_expr(code)
        self.body.generate_execution_code(code)
        self.teardown_temp_expr(code)

    def generate_function_definitions(self, env, code):
        self.temp_expression.generate_function_definitions(env, code)
        self.body.generate_function_definitions(env, code)


class TempResultFromStatNode(ExprNodes.ExprNode):
    # An ExprNode wrapper around a StatNode that executes the StatNode
    # body.  Requires a ResultRefNode that it sets up to refer to its
    # own temp result.  The StatNode must assign a value to the result
    # node, which then becomes the result of this node.

    subexprs = []
    child_attrs = ['body']

    def __init__(self, result_ref, body):
        self.result_ref = result_ref
        self.pos = body.pos
        self.body = body
        self.type = result_ref.type
        self.is_temp = 1

    def analyse_declarations(self, env):
        self.body.analyse_declarations(env)

    def analyse_types(self, env):
        self.body = self.body.analyse_expressions(env)
        return self

    def may_be_none(self):
        return self.result_ref.may_be_none()

    def generate_result_code(self, code):
        self.result_ref.result_code = self.result()
        self.body.generate_execution_code(code)

    def generate_function_definitions(self, env, code):
        self.body.generate_function_definitions(env, code)


class HasNoGilNode(AtomicExprNode):
    """
    Simple node that evaluates to
    * 0 if gil
    * 1 if nogil
    * 2 if maybe gil
    """
    type = c_int_type

    def analyse_types(self, env):
        return self

    def generate_result_code(self, code):
        pass

    def calculate_result_code(self):
        return str(int(self.in_nogil_context))



cdef extern from *:
    cpdef enum: # ExternPxd
        FOUR "4"
        EIGHT "8"

    cdef enum: # ExternSecretPxd
        SIXTEEN "16"

cpdef enum PxdEnum:
    RANK_0 = 11
    RANK_1 = 37
    RANK_2 = 389

cpdef enum cpdefPxdDocEnum:
    """Home is where...
    """
    RANK_6 = 159

cpdef enum cpdefPxdDocLineEnum:
    """Home is where..."""
    RANK_7 = 889

cdef enum PxdSecretEnum:
    RANK_8 = 5077

cdef enum cdefPxdDocEnum:
    """the heart is.
    """
    RANK_9 = 2458



# tag: cpp
# mode: run, no-cpp-locals

cdef extern from *:
    """
    enum Enum1 {
        Item1,
        Item2
    };

    """
    cdef enum Enum1:
        Item1
        Item2

a = Item1
b = Item2

cdef Enum1 x, y
x = Item1
y = Item2


def compare_enums():
    """
    >>> compare_enums()
    (True, True, True, True)
    """
    return x == a, a == Item1, b == y, y == Item2


cdef extern from * namespace "Namespace1":
    """
    namespace Namespace1 {
        enum Enum2 {
            Item3,
            Item4
        };
    }
    """
    cdef enum Enum2:
        Item3
        Item4

c = Item3
d = Item4

cdef Enum2 z, w
z = Item3
w = Item4


def compare_namespace_enums():
    """
    >>> compare_namespace_enums()
    (True, True, True, True)
    """
    return z == c, c == Item3, d == w, d == Item4



# mode: run
# tag: pyclass

class A(object):
    x = 1

class B(object):
    x = 2


def cond_if_bases(x):
    """
    >>> c = cond_if_bases(True)
    >>> c().p
    5
    >>> c().x
    1
    >>> c = cond_if_bases(False)
    >>> c().p
    5
    >>> c().x
    2
    """
    class PyClass(A if x else B):
        p = 5
    return PyClass


def make_subclass(*bases):
    """
    >>> cls = make_subclass(list)
    >>> issubclass(cls, list) or cls.__mro__
    True

    >>> class Cls(object): pass
    >>> cls = make_subclass(Cls, list)
    >>> issubclass(cls, list) or cls.__mro__
    True
    >>> issubclass(cls, Cls) or cls.__mro__
    True
    """
    # GH-3338
    class MadeClass(*bases):
        pass
    return MadeClass



import numpy as np

def main():

    to_view: cython.int[:, :, :] = np.empty((20, 15, 30), dtype=np.intc)
    from_view: cython.int[:, :, :] = np.ones((20, 15, 30), dtype=np.intc)

    # copy the elements in from_view to to_view
    to_view[...] = from_view
    # or
    to_view[:] = from_view
    # or
    to_view[:, :, :] = from_view



"""
>>> import sys

>>> ANON_A, ANON_B, ANON_X
(2, 3, 5)

>>> ONE, TEN, HUNDRED
(1, 10, 100)
>>> THOUSAND        # doctest: +ELLIPSIS
Traceback (most recent call last):
NameError: ...name 'THOUSAND' is not defined

>>> PyxEnum.TWO == 2 or PyxEnum.TWO
True
>>> PyxEnum.THREE == 3 or PyxEnum.THREE
True
>>> PyxEnum.FIVE == 5 or PyxEnum.FIVE
True
>>> cpdefPyxDocEnum.ELEVEN == 11 or cpdefPyxDocEnum.ELEVEN
True

>>> SEVEN           # doctest: +ELLIPSIS
Traceback (most recent call last):
NameError: ...name 'SEVEN' is not defined
>>> SecretPyxEnum   # doctest: +ELLIPSIS
Traceback (most recent call last):
NameError: ...name 'SecretPyxEnum' is not defined
>>> PxdSecretEnum   # doctest: +ELLIPSIS
Traceback (most recent call last):
NameError: ...name 'PxdSecretEnum' is not defined
>>> cdefPxdDocEnum   # doctest: +ELLIPSIS
Traceback (most recent call last):
NameError: ...name 'cdefPxdDocEnum' is not defined

>>> FOUR == 4 or FOUR
True
>>> EIGHT == 8 or EIGHT
True
>>> SIXTEEN        # doctest: +ELLIPSIS
Traceback (most recent call last):
NameError: ...name 'SIXTEEN' is not defined

>>> RANK_0         # doctest: +ELLIPSIS
Traceback (most recent call last):
NameError: ...name 'RANK_0' is not defined

>>> PxdEnum.RANK_0 == 11 or PxdEnum.RANK_0
True
>>> PxdEnum.RANK_1 == 37 or PxdEnum.RANK_1
True
>>> PxdEnum.RANK_2 == 389 or PxdEnum.RANK_2
True
>>> cpdefPxdDocEnum.RANK_6 == 159 or cpdefPxdDocEnum.RANK_6
True
>>> cpdefPxdDocLineEnum.RANK_7 == 889 or cpdefPxdDocLineEnum.RANK_7
True
>>> PxdEnum.RANK_3         # doctest: +ELLIPSIS
Traceback (most recent call last):
AttributeError: ...RANK_3...

>>> set(PyxEnum) == {PyxEnum.TWO, PyxEnum.THREE, PyxEnum.FIVE}
True
>>> str(PyxEnum.TWO).split(".")[-1]  if sys.version_info < (3,11) else  "TWO" # Py3.10/11 changed the output here
'TWO'
>>> str(PyxEnum.TWO)  if sys.version_info >= (3,11) else  "2" # Py3.10/11 changed the output here
'2'
>>> PyxEnum.TWO + PyxEnum.THREE == PyxEnum.FIVE
True
>>> PyxEnum(2) is PyxEnum["TWO"] is PyxEnum.TWO
True

# not leaking into module namespace
>>> IntEnum        # doctest: +ELLIPSIS
Traceback (most recent call last):
NameError: ...name 'IntEnum' is not defined
"""

cdef extern from *:
    cpdef enum: # ExternPyx
        ONE "1"
        TEN "10"
        HUNDRED "100"

    cdef enum: # ExternSecretPyx
        THOUSAND "1000"

cpdef enum:
    # Anonymous enum values should appear in the module namespace.
    ANON_A = 2
    ANON_B = 3
    ANON_X = 5

cpdef enum PyxEnum:
    TWO = 2
    THREE = 3
    FIVE = 5

cpdef enum cpdefPyxDocEnum:
    """Home is where...
    """
    ELEVEN = 11

cpdef enum cpdefPyxDocLineEnum:
    """Home is where..."""
    FOURTEEN = 14

cdef enum SecretPyxEnum:
    SEVEN = 7

cdef enum cdefPyxDocEnum:
    """the heart is.
    """
    FIVE_AND_SEVEN = 5077

cdef extern from *:
    """
    enum ExternHasDuplicates {
        EX_DUP_A,
        EX_DUP_B=EX_DUP_A,
        EX_DUP_C=EX_DUP_A
    };
    """
    # Cython doesn't know about the duplicates though
    cpdef enum ExternHasDuplicates:
        EX_DUP_A
        EX_DUP_B
        EX_DUP_C


cpdef enum CyDefinedHasDuplicates1:
    CY_DUP1_A
    CY_DUP1_B = 0x00000000


cpdef enum CyDefinedHasDuplicates2:
    CY_DUP2_A
    CY_DUP2_B = CY_DUP2_A

cpdef enum CyDefinedHasDuplicates3:
    CY_DUP3_A = 1
    CY_DUP3_B = 0
    CY_DUP3_C  # = 1


def test_as_variable_from_cython():
    """
    >>> test_as_variable_from_cython()
    """
    assert list(PyxEnum) == [TWO, THREE, FIVE], list(PyxEnum)
    assert list(PxdEnum) == [RANK_0, RANK_1, RANK_2], list(PxdEnum)

cdef int verify_pure_c() nogil:
    cdef int x = TWO
    cdef int y = PyxEnum.THREE
    cdef int z = SecretPyxEnum.SEVEN
    return x + y + z

# Use it to suppress warning.
verify_pure_c()

def verify_resolution_GH1533():
    """
    >>> verify_resolution_GH1533()
    3
    """
    THREE = 100
    return int(PyxEnum.THREE)


def check_docs():
    """
    >>> PxdEnum.__doc__ not in ("Home is where...\\n    ", "Home is where...")
    True
    >>> PyxEnum.__doc__ not in ("Home is where...\\n    ", "Home is where...")
    True
    >>> cpdefPyxDocEnum.__doc__ == "Home is where...\\n    "
    True
    >>> cpdefPxdDocEnum.__doc__ == "Home is where...\\n    "
    True
    >>> cpdefPyxDocLineEnum.__doc__
    'Home is where...'
    >>> cpdefPxdDocLineEnum.__doc__
    'Home is where...'
    """
    pass


def to_from_py_conversion(PxdEnum val):
    """
    >>> to_from_py_conversion(PxdEnum.RANK_1) is PxdEnum.RANK_1
    True

    C enums are commonly enough used as flags that it seems reasonable
    to allow it in Cython
    >>> to_from_py_conversion(PxdEnum.RANK_1 | PxdEnum.RANK_2) == (PxdEnum.RANK_1 | PxdEnum.RANK_2)
    True
    """
    return val


def to_from_py_conversion_with_duplicates1(ExternHasDuplicates val):
    """
    Mainly a compile-time test - we can't optimize to a switch here
    >>> to_from_py_conversion_with_duplicates1(ExternHasDuplicates.EX_DUP_A) == ExternHasDuplicates.EX_DUP_A
    True
    """
    return val


def to_from_py_conversion_with_duplicates2(CyDefinedHasDuplicates1 val):
    """
    Mainly a compile-time test - we can't optimize to a switch here
    >>> to_from_py_conversion_with_duplicates2(CyDefinedHasDuplicates1.CY_DUP1_A) == CyDefinedHasDuplicates1.CY_DUP1_A
    True
    """
    return val


def to_from_py_conversion_with_duplicates3(CyDefinedHasDuplicates2 val):
    """
    Mainly a compile-time test - we can't optimize to a switch here
    >>> to_from_py_conversion_with_duplicates3(CyDefinedHasDuplicates2.CY_DUP2_A) == CyDefinedHasDuplicates2.CY_DUP2_A
    True
    """
    return val


def to_from_py_conversion_with_duplicates4(CyDefinedHasDuplicates3 val):
    """
    Mainly a compile-time test - we can't optimize to a switch here
    >>> to_from_py_conversion_with_duplicates4(CyDefinedHasDuplicates3.CY_DUP3_C) == CyDefinedHasDuplicates3.CY_DUP3_C
    True
    """
    return val


def test_pickle():
    """
    >>> from pickle import loads, dumps
    >>> import sys

    Python 3.11.4 has a bug that breaks pickling: https://github.com/python/cpython/issues/105332

    >>> if sys.version_info[:3] == (3,11,4):
    ...     loads = dumps = lambda x: x

    >>> loads(dumps(PyxEnum.TWO)) == PyxEnum.TWO
    True
    >>> loads(dumps(PxdEnum.RANK_2)) == PxdEnum.RANK_2
    True
    """
    pass

def test_as_default_value(PxdEnum val=PxdEnum.RANK_1):
    """
    In order to work, this requires the utility code to be evaluated
    before the function definition
    >>> test_as_default_value()
    True
    >>> test_as_default_value(PxdEnum.RANK_2)
    False
    >>> test_as_default_value.__defaults__[0] == PxdEnum.RANK_1
    True
    """
    return val == PxdEnum.RANK_1


def test_special_attributes():
    """
    >>> test_special_attributes()
    ('cpdefPyxDocLineEnum', 'cpdef_enums', 'Home is where...')
    """
    assert PyxEnum.__contains__
    assert PyxEnum.__init_subclass__
    assert PyxEnum.__iter__
    assert PyxEnum.__len__
    assert PyxEnum.__class__

    assert PyxEnum.__getitem__('TWO') is PyxEnum.TWO
    assert PyxEnum.__members__['TWO'] is PyxEnum.TWO
    assert PyxEnum.__qualname__.endswith(PyxEnum.__name__), (PyxEnum.__qualname__, PyxEnum.__name__)

    return (
        cpdefPyxDocLineEnum.__name__,
        cpdefPyxDocLineEnum.__module__,
        cpdefPyxDocLineEnum.__doc__,
    )



cdef int CHKERR(int ierr) except -1:
    if ierr==0: return 0
    raise RuntimeError

cdef int obj2int(object ob) except *:
    return ob

def foo(a):
    """
    >>> foo(0)
    >>> foo(1)
    Traceback (most recent call last):
    RuntimeError
    """
    cdef int i = obj2int(a)
    CHKERR(i)

cdef int* except_expr(bint fire) except <int*>-1:
    if fire:
        raise RuntimeError

def test_except_expr(bint fire):
    """
    >>> test_except_expr(False)
    >>> test_except_expr(True)
    Traceback (most recent call last):
    ...
    RuntimeError
    """
    except_expr(fire)

cdef double except_big_result(bint fire) except 100000000000000000000000000000000:
    if fire:
        raise RuntimeError

def test_except_big_result(bint fire):
    """
    >>> test_except_big_result(False)
    >>> test_except_big_result(True)
    Traceback (most recent call last):
    ...
    RuntimeError
    """
    except_big_result(fire)


cdef unsigned short except_promotion_compare(bint fire) except *:
    if fire:
        raise RuntimeError

def test_except_promotion_compare(bint fire):
    """
    >>> test_except_promotion_compare(False)
    >>> test_except_promotion_compare(True)
    Traceback (most recent call last):
    ...
    RuntimeError
    """
    except_promotion_compare(fire)


cdef int cdef_function_that_raises():
    raise RuntimeError

cdef int cdef_noexcept_function_that_raises() noexcept:
    raise RuntimeError

def test_except_raise_by_default():
    """
    >>> test_except_raise_by_default()
    Traceback (most recent call last):
    ...
    RuntimeError
    """
    cdef_function_that_raises()

def test_noexcept():
    """
    >>> test_noexcept()
    """
    cdef_noexcept_function_that_raises()


cdef int* cdef_ptr_func(int* input, int failure_mode):
    # should have except NULL? by default
    # failure mode is 0, 1, or 2
    if failure_mode == 0:
        return input  # don't fail
    elif failure_mode == 1:
        return NULL  # no exception
    else:
        raise RuntimeError("help!")

ctypedef int* (*cdef_ptr_func_ptr)(int*, int) except? NULL

def test_ptr_func(int failure_mode):
    """
    >>> test_ptr_func(0)
    100
    >>> test_ptr_func(1)
    NULL
    >>> test_ptr_func(2)
    exception
    """
    # check that the signature is what we think it is
    cdef cdef_ptr_func_ptr fptr = cdef_ptr_func
    cdef int a = 100
    try:
        out = fptr(&a, failure_mode)
        if out:
            return out[0]
        else:
            print("NULL")
    except RuntimeError:
        print("exception")

def test_ptr_func2(int failure_mode):
    """
    >>> test_ptr_func(0)
    100
    >>> test_ptr_func(1)
    NULL
    >>> test_ptr_func(2)
    exception
    """
    # as above, but don't go through a function pointer
    cdef int a = 100
    try:
        out = cdef_ptr_func(&a, failure_mode)
        if out:
            return out[0]
        else:
            print("NULL")
    except RuntimeError:
        print("exception")

cdef double return_double(double arg, bint fail):
    if fail:
        raise RuntimeError
    return arg

ctypedef double (*func_ptr1)(double, bint) except? -1

def test_return_double(fail):
    """
    >>> test_return_double(False)
    2.0
    >>> test_return_double(True)
    exception
    """
    # Test that we can assign to the function pointer we expect
    # https://github.com/cython/cython/issues/5709 - representation of the "-1" was fragile
    cdef func_ptr1 p1 = return_double
    try:
        return p1(2.0, fail)
    except RuntimeError:
        print("exception")

cdef enum E:
    E1
    E2

cdef E return_enum1(bint fail) except? E.E1:
    if fail:
        raise RuntimeError
    return E.E2

cdef E return_enum2(bint fail) except? E1:
    if fail:
        raise RuntimeError
    return E.E2

def test_enum1(fail):
    """
    >>> test_enum1(False)
    True
    >>> test_enum1(True)
    exception
    """
    try:
        return return_enum1(fail) == E2
    except RuntimeError:
        print("exception")

def test_enum2(fail):
    """
    >>> test_enum2(False)
    True
    >>> test_enum2(True)
    exception
    """
    try:
        return return_enum2(fail) == E2
    except RuntimeError:
        print("exception")

cdef char return_char(fail) except 'a':
    if fail:
        raise RuntimeError
    return 'b'

def test_return_char(fail):
    """
    >>> test_return_char(False)
    'b'
    >>> test_return_char(True)
    exception
    """
    try:
        return chr(return_char(fail))
    except RuntimeError:
        print("exception")



# mode: run
# ticket: t600
# tag: genexpr
# cython: language_level=3

cimport cython

#@cython.test_assert_path_exists('//ComprehensionNode')
#@cython.test_fail_if_path_exists('//SimpleCallNode')
def list_genexpr_iterable_lookup():
    """
    >>> x = (0,1,2,3,4,5)
    >>> [ x*2 for x in x if x % 2 == 0 ]  # leaks in Py2 but finds the right 'x'
    [0, 4, 8]

    >>> list_genexpr_iterable_lookup()
    [0, 4, 8]
    """
    x = (0,1,2,3,4,5)
    result = list( x*2 for x in x if x % 2 == 0 )
    assert x == (0,1,2,3,4,5)
    return result


#@cython.test_assert_path_exists('//ComprehensionNode')
#@cython.test_fail_if_path_exists('//SingleAssignmentNode//SimpleCallNode')
def genexpr_iterable_in_closure():
    """
    >>> genexpr_iterable_in_closure()
    ['aa', 'cc']
    """
    x = 'abc'
    def f():
        return x
    result = list( x*2 for x in x if x != 'b' )
    assert x == 'abc' # don't leak in Py3 code
    assert f() == 'abc' # don't leak in Py3 code

    return result


def genexpr_over_complex_arg(func, L):
    """
    >>> class wrapper(object):
    ...     value = 5
    >>> genexpr_over_complex_arg(list, wrapper())
    [5]
    """
    return func(d for d in set([type(L).value, L.__class__.value, L.value]))


def listcomp():
    """
    >>> listcomp()
    [0, 1, 5, 8]
    """
    data = [('red', 5), ('blue', 1), ('yellow', 8), ('black', 0)]
    data.sort(key=lambda r: r[1])
    keys = [r[1] for r in data]
    return keys


def genexpr_in_listcomp(L):
    """
    >>> genexpr_in_listcomp( [[1,2,3]]*2 )
    [[1, 2, 3], [1, 2, 3]]
    """
    return list(d for d in [list(d for d in d) for d in L])


@cython.test_assert_path_exists('//ForFromStatNode')
def genexpr_range_in_listcomp(L):
    """
    >>> genexpr_range_in_listcomp( [1,2,3] )
    [[0], [0, 1], [0, 1, 2]]
    """
    cdef int z,d
    return [list(d for d in range(z)) for z in L]


@cython.test_fail_if_path_exists('//ForInStatNode')
def genexpr_in_dictcomp_dictiter():
    """
    >>> sorted(genexpr_in_dictcomp_dictiter())
    [1, 5]
    """
    d = {1:2, 3:4, 5:6}
    return {k:d for k,d in d.iteritems() if d != 4}


def genexpr_over_array_slice():
    """
    >>> list(genexpr_over_array_slice())
    [0.0, 1.0, 2.0, 3.0, 4.0, 5.0]
    """
    cdef double x[10]
    for i in range(10):
        x[i] = i
    cdef int n = 5
    return (n for n in x[:n+1])



# mode: run
# tag: closures
# ticket: t658

def outer(int x, *args, **kwargs):
    """
    >>> inner = outer(1, 2, a=3)
    >>> inner()
    (1, (2,), {'a': 3})

    >>> inner = outer('abc', 2, a=3)
    Traceback (most recent call last):
    TypeError: an integer is required
    """
    def inner():
        return x, args, kwargs
    return inner



# mode: run
# tag: cpp, werror, cpp11

from cython.operator cimport dereference as deref

from libcpp cimport bool
from libcpp.algorithm cimport all_of, any_of, none_of, for_each, count, count_if, mismatch, find, find_if, find_if_not
from libcpp.algorithm cimport find_end, find_first_of, adjacent_find, search, search_n
from libcpp.iterator cimport distance
from libcpp.string cimport string
from libcpp.vector cimport vector


cdef bool is_odd(int i):
    return i % 2


def all_odd(vector[int] values):
    """
    Test all_of with is_odd predicate.

    >>> all_odd([3, 5, 7, 11, 13, 17, 19, 23])
    True
    >>> all_odd([3, 4])
    False
    """
    return all_of(values.begin(), values.end(), is_odd)


def any_odd(vector[int] values):
    """
    Test any_of with is_odd predicate.

    >>> any_odd([1, 2, 3, 4])
    True
    >>> any_odd([2, 4, 6, 8])
    False
    """
    return any_of(values.begin(), values.end(), is_odd)


def none_odd(vector[int] values):
    """
    Test none_of with is_odd predicate.

    >>> none_odd([2, 4, 6, 8])
    True
    >>> none_odd([1, 2, 3, 4])
    False
    """
    return none_of(values.begin(), values.end(), is_odd)


def count_ones(vector[int] values):
    """
    Test count.

    >>> count_ones([1, 2, 1, 2])
    2
    """
    return count(values.begin(), values.end(), 1)


cdef void add_one(int &i):
    # https://github.com/cython/cython/issues/1863
    (&i)[0] += 1


def increment_ints(vector[int] values):
    """
    Test for_each.

    >>> increment_ints([3, 4, 2, 8, 15, 267])
    [4, 5, 3, 9, 16, 268]
    """
    for_each(values.begin(), values.end(), &add_one)
    return values


def count_odd(vector[int] values):
    """
    Test count_if with is_odd predicate.

    >>> count_odd([1, 2, 3, 4])
    2
    >>> count_odd([2, 4, 6, 8])
    0
    """
    return count_if(values.begin(), values.end(), is_odd)


def mirror_ends(string data):
    """
    Test mismatch using cppreference example.

    This program determines the longest substring that is simultaneously found at the very beginning of the given string
    and at the very end of it, in reverse order (possibly overlapping).

    >>> print(mirror_ends(b'abXYZba').decode('ascii'))
    ab
    >>> print(mirror_ends(b'abca').decode('ascii'))
    a
    >>> print(mirror_ends(b'aba').decode('ascii'))
    aba
    """
    return string(data.begin(), mismatch(data.begin(), data.end(), data.rbegin()).first)


def mismatch_ints(vector[int] values1, vector[int] values2):
    """
    Test mismatch(first1, last1, first2).

    >>> mismatch_ints([1, 2, 3], [1, 2, 3])
    >>> mismatch_ints([1, 2], [1, 2, 3])
    >>> mismatch_ints([1, 3], [1, 2, 3])
    (3, 2)
    """
    result = mismatch(values1.begin(), values1.end(), values2.begin())
    if result.first == values1.end():
        return
    return deref(result.first), deref(result.second)


def is_int_in(vector[int] values, int target):
    """
    Test find.

    >>> is_int_in(range(5), 3)
    True
    >>> is_int_in(range(5), 10)
    False
    """
    return find(values.begin(), values.end(), target) != values.end()


def find_odd(vector[int] values):
    """
    Test find_if using is_odd predicate.

    >>> find_odd([2, 3, 4])
    3
    >>> find_odd([2, 4, 6])
    """
    result = find_if(values.begin(), values.end(), is_odd)
    if result != values.end():
        return deref(result)
    else:
        return None


def find_even(vector[int] values):
    """
    Test find_if_not using is_odd predicate.

    >>> find_even([3, 4, 5])
    4
    >>> find_even([1, 3, 5])
    """
    result = find_if_not(values.begin(), values.end(), is_odd)
    if result != values.end():
        return deref(result)
    else:
        return None


def find_last_int_sequence(vector[int] values, vector[int] target):
    """
    Test find_end.

    >>> find_last_int_sequence([1, 2, 3, 1, 2, 3], [2, 3])
    4
    >>> find_last_int_sequence([1, 2, 3], [4, 5])
    """
    result = find_end(values.begin(), values.end(), target.begin(), target.end())
    if result != values.end():
        return distance(values.begin(), result)
    else:
        return None


cdef bool is_equal(int lhs, int rhs):
    return lhs == rhs


def find_last_int_sequence2(vector[int] values, vector[int] target):
    """
    Test find_end (using is_equal predicate).

    >>> find_last_int_sequence2([1, 2, 3, 1, 2, 3], [2, 3])
    4
    >>> find_last_int_sequence2([1, 2, 3], [4, 5])
    """
    result = find_end(values.begin(), values.end(), target.begin(), target.end(), &is_equal)
    if result != values.end():
        return distance(values.begin(), result)
    else:
        return None


def find_first_int_in_set(values, target):
    """
    Test find_first_of.

    >>> find_first_int_in_set([1, 2, 3, 4, 5], [3, 5])
    2
    >>> find_first_int_in_set([1, 2, 3], [4, 5])
    """
    cdef vector[int] v = values
    cdef vector[int] t = target
    result = find_first_of(v.begin(), v.end(), t.begin(), t.end())
    if result != v.end():
        return distance(v.begin(), result)
    else:
        return None


def find_first_int_in_set2(vector[int] values, vector[int] target):
    """
    Test find_first_of with is_equal predicate.

    >>> find_first_int_in_set2([1, 2, 3, 4, 5], [3, 5])
    2
    >>> find_first_int_in_set2([1, 2, 3], [4, 5])
    """
    result = find_first_of(values.begin(), values.end(), target.begin(), target.end(), is_equal)
    if result != values.end():
        return distance(values.begin(), result)
    else:
        return None


def find_adjacent_int(vector[int] values):
    """
    Test adjacent_find.

    >>> find_adjacent_int([0, 1, 2, 3, 40, 40, 41, 41, 5])
    4
    >>> find_adjacent_int(range(5))
    """
    result = adjacent_find(values.begin(), values.end())
    if result != values.end():
        return distance(values.begin(), result)
    else:
        return None


def find_adjacent_int2(vector[int] values):
    """
    Test find_adjacent with is_equal predicate.

    >>> find_adjacent_int2([0, 1, 2, 3, 40, 40, 41, 41, 5])
    4
    >>> find_adjacent_int2(range(5))
    """
    result = adjacent_find(values.begin(), values.end(), is_equal)
    if result != values.end():
        return distance(values.begin(), result)
    else:
        return None


def in_quote(string quote, string word):
    """
    Test search using cppreference example.

    >>> in_quote(b"why waste time learning, when ignorance is instantaneous?", b"learning")
    True
    >>> in_quote(b"why waste time learning, when ignorance is instantaneous?", b"lemming")
    False
    """
    return search(quote.begin(), quote.end(), word.begin(), word.end()) != quote.end()


def in_quote2(string quote, string word):
    """
    Test search using cppreference example (with is_equal predicate).

    >>> in_quote2(b"why waste time learning, when ignorance is instantaneous?", b"learning")
    True
    >>> in_quote2(b"why waste time learning, when ignorance is instantaneous?", b"lemming")
    False
    """
    return search(quote.begin(), quote.end(), word.begin(), word.end(), &is_equal) != quote.end()


def consecutive_values(string c, int count, char v):
    """
    Test search_n using cppreference example (without std::begin and std::end).

    >>> consecutive_values(b"1001010100010101001010101", 4, ord("0"))
    False
    >>> consecutive_values(b"1001010100010101001010101", 3, ord("0"))
    True
    """
    return search_n(c.begin(), c.end(), count, v) != c.end()


def consecutive_values2(string c, int count, char v):
    """
    Test search_n using cppreference example (with is_equal predicate).

    >>> consecutive_values2(b"1001010100010101001010101", 4, ord("0"))
    False
    >>> consecutive_values2(b"1001010100010101001010101", 3, ord("0"))
    True
    """
    return search_n(c.begin(), c.end(), count, v, &is_equal) != c.end()



# mode: compile

__doc__ = u"""
    >>> s = Swallow()
    >>> s.spam(1)
    1 42 'grail' True
    >>> s.spam(1, 2)
    1 2 'grail' True
    >>> s.spam(1, z = 2)
    1 42 'grail' 2
    >>> s.spam(1, y = 2)
    1 42 2 True
    >>> s.spam(1, x = 2, y = 'test')
    1 2 'test' True
"""

swallow = True

class Swallow:

    def spam(w, int x = 42, y = "grail", z = swallow):
        print w, x, y, z



from cython.cimports.cpython import array
import array

a = cython.declare(array.array, array.array('i', [1, 2, 3]))

# access underlying pointer:
print(a.data.as_ints[0])

from cython.cimports.libc.string import memset

memset(a.data.as_voidptr, 0, len(a) * cython.sizeof(cython.int))



cimport cython

from libc.stdint cimport uint64_t

class A:
    def pop(self, *args):
        print args
        return None

cdef class B:
    """
    >>> B().call_pop()
    'B'
    """
    cdef pop(self):
        return "B"
    def call_pop(self):
        return self.pop()


@cython.test_assert_path_exists('//PythonCapiCallNode')
@cython.test_fail_if_path_exists('//SimpleCallNode/AttributeNode')
def simple_pop(L):
    """
    >>> L = list(range(10))
    >>> simple_pop(L)
    9
    >>> simple_pop(L)
    8
    >>> L
    [0, 1, 2, 3, 4, 5, 6, 7]
    >>> while L:
    ...    _ = simple_pop(L)

    >>> L
    []
    >>> simple_pop(L)
    Traceback (most recent call last):
    IndexError: pop from empty list

    >>> simple_pop(A())
    ()
    """
    return L.pop()

@cython.test_assert_path_exists('//PythonCapiCallNode')
@cython.test_fail_if_path_exists('//SimpleCallNode/AttributeNode')
def simple_pop_typed(list L):
    """
    >>> L = list(range(10))
    >>> simple_pop_typed(L)
    9
    >>> simple_pop_typed(L)
    8
    >>> L
    [0, 1, 2, 3, 4, 5, 6, 7]
    >>> while L:
    ...    _ = simple_pop_typed(L)

    >>> L
    []
    >>> simple_pop_typed(L)
    Traceback (most recent call last):
    IndexError: pop from empty list
    """
    return L.pop()


@cython.test_assert_path_exists('//PythonCapiCallNode')
@cython.test_fail_if_path_exists('//SimpleCallNode/AttributeNode')
def index_pop(L, int i):
    """
    >>> L = list(range(10))
    >>> index_pop(L, 2)
    2
    >>> index_pop(L, -10)
    Traceback (most recent call last):
    IndexError: pop index out of range
    >>> index_pop(L, -2)
    8
    >>> L
    [0, 1, 3, 4, 5, 6, 7, 9]
    >>> index_pop(L, 100)
    Traceback (most recent call last):
    IndexError: pop index out of range
    >>> index_pop(L, -100)
    Traceback (most recent call last):
    IndexError: pop index out of range

    >>> while L:
    ...    _ = index_pop(L, 0)

    >>> L
    []

    >>> index_pop(L, 0)
    Traceback (most recent call last):
    IndexError: pop from empty list

    >>> index_pop(A(), 3)
    (3,)
    """
    return L.pop(i)

@cython.test_assert_path_exists('//PythonCapiCallNode')
@cython.test_fail_if_path_exists('//SimpleCallNode/AttributeNode')
def index_pop_typed(list L, int i):
    """
    >>> L = list(range(10))
    >>> index_pop_typed(L, 2)
    2
    >>> index_pop_typed(L, -2)
    8
    >>> L
    [0, 1, 3, 4, 5, 6, 7, 9]
    >>> index_pop_typed(L, 100)
    Traceback (most recent call last):
    IndexError: pop index out of range
    >>> index_pop_typed(L, -100)
    Traceback (most recent call last):
    IndexError: pop index out of range

    >>> index_pop_typed(None, 0)
    Traceback (most recent call last):
    AttributeError: 'NoneType' object has no attribute 'pop'

    >>> while L:
    ...    _ = index_pop_typed(L, 0)

    >>> L
    []

    >>> index_pop_typed(L, 0)
    Traceback (most recent call last):
    IndexError: pop from empty list
    """
    return L.pop(i)


@cython.test_assert_path_exists('//PythonCapiCallNode')
@cython.test_fail_if_path_exists('//SimpleCallNode/AttributeNode')
def index_pop_list_object_index(list L, i):
    """
    >>> L = list(range(10))
    >>> index_pop_list_object_index(L, 2)
    2
    >>> index_pop_list_object_index(L, -2)
    8
    >>> L
    [0, 1, 3, 4, 5, 6, 7, 9]
    >>> index_pop_list_object_index(L, 100)
    Traceback (most recent call last):
    IndexError: pop index out of range
    >>> index_pop_list_object_index(L, -100)
    Traceback (most recent call last):
    IndexError: pop index out of range

    >>> index_pop_list_object_index(None, 0)
    Traceback (most recent call last):
    AttributeError: 'NoneType' object has no attribute 'pop'
    >>> index_pop_list_object_index([1], None)    # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...
    >>> index_pop_list_object_index([1], 'abc')   # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...

    >>> while L:
    ...    _ = index_pop_list_object_index(L, 0)

    >>> L
    []

    >>> index_pop_list_object_index(L, 0)
    Traceback (most recent call last):
    IndexError: pop from empty list
    """
    return L.pop(i)


@cython.test_assert_path_exists('//PythonCapiCallNode')
@cython.test_fail_if_path_exists('//SimpleCallNode/AttributeNode')
def index_pop_literal(list L):
    """
    >>> L = list(range(10))
    >>> index_pop_literal(L)
    0
    >>> L
    [1, 2, 3, 4, 5, 6, 7, 8, 9]

    >>> while L:
    ...    _ = index_pop_literal(L)

    >>> L
    []

    >>> index_pop_literal(L)
    Traceback (most recent call last):
    IndexError: pop from empty list
    """
    return L.pop(0)


@cython.test_fail_if_path_exists('//PythonCapiCallNode')
def crazy_pop(L):
    """
    >>> crazy_pop(list(range(10)))    # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError: ...pop... argument...
    >>> crazy_pop(A())
    (1, 2, 3)
    """
    return L.pop(1, 2, 3)


def method_name():
    """
    >>> method_name()
    'pop'
    """
    return [].pop.__name__


def object_pop_large_int():
    """
    >>> object_pop_large_int()
    {}
    """
    cdef object foo = {}
    cdef uint64_t bar = 201213467776703617ULL

    foo[bar] = None
    assert (<object>bar) in foo
    foo.pop(bar)
    return foo



# mode: compile

# This is a sort of meta test - to test the functionality of "test_assert_path_exists"

cimport cython

@cython.test_assert_path_exists("//ReturnStatNode")
def not_in_inner_compiler_directives():
    # used to fail because ReturnStatNode wasn't in *this* CompilerDirectivesNode
    with cython.boundscheck(False):
        pass
    return 1 # should pass

@cython.test_assert_path_exists("//ReturnStatNode")
def in_inner_compiler_directives():
    # used to fail because ReturnStatNode wasn't in *this* CompilerDirectivesNode
    with cython.boundscheck(False):
        return 1

# it's hard to come up with a corresponding test for fail_if_path_exists..



# ticket: t601

cdef unsigned long size2():
    return 3

def for_from_plain_ulong():
    """
    >>> for_from_plain_ulong()
    0
    1
    2
    """
    cdef object j = 0
    for j from 0 <= j < size2():
        print j

def for_in_plain_ulong():
    """
    >>> for_in_plain_ulong()
    0
    1
    2
    """
    cdef object j = 0
    for j in range(size2()):
        print j


cdef extern from *:
    """typedef unsigned long Ulong;"""
    ctypedef unsigned long Ulong

cdef Ulong size():
    return 3

def for_from_ctypedef_ulong():
    """
    >>> for_from_ctypedef_ulong()
    0
    1
    2
    """
    cdef object j = 0
    for j from 0 <= j < size():
        print j

def for_in_ctypedef_ulong():
    """
    >>> for_in_ctypedef_ulong()
    0
    1
    2
    """
    cdef object j = 0
    for j in range(size()):
        print j


class ForFromLoopInPyClass(object):
    """
    >>> ForFromLoopInPyClass.i    # doctest: +ELLIPSIS
    Traceback (most recent call last):
    AttributeError: ...ForLoopInPyClass... has no attribute ...i...
    >>> ForFromLoopInPyClass.k
    0
    >>> ForFromLoopInPyClass.m
    1
    """
    for i from 0 <= i < 1:
        pass

    for k from 0 <= k < 2:
        pass

    for m from 0 <= m < 3:
        pass



# mode: compile

from __future__ import nested_scopes

from __future__ import with_statement

pass

from __future__ import nested_scopes ; from __future__ import nested_scopes

from __future__ import print_function
print(end='')



# mode: run
# tag: warnings

from __future__ import print_function

cimport cython


def pow_double_double(double a, double b, delta):
    """
    >>> pow_double_double(2, 2, 1e-15)
    soft double complex float
    >>> pow_double_double(4, 0.5, 1e-15)
    soft double complex float
    >>> pow_double_double(-4, 0.5, 1e-15)
    soft double complex complex
    """
    c = a**b
    # print out the Cython type, and the coerced type
    print(cython.typeof(c), type(c).__name__)
    object_c = (<object>a)**(<object>b)
    assert abs((c/object_c) - 1) < delta

@cython.cpow(True)
def pow_double_double_cpow(double a, double b, delta=None):
    """
    >>> pow_double_double_cpow(2, 2, 1e-15)
    double float
    >>> pow_double_double_cpow(4, 0.5, 1e-15)
    double float
    >>> x = pow_double_double_cpow(-4, 0.5)
    double float
    >>> x == x  # is nan
    False
    """
    c = a**b
    # print out the Cython type, and the coerced type
    print(cython.typeof(c), type(c).__name__)
    if delta is not None:
        object_c = (<object>a)**(<object>b)
        assert abs((c/object_c) - 1) < delta
    else:
        return c

cdef cfunc_taking_double(double x):
    return x

def pow_double_double_coerced_directly(double a, double b):
    """
    >>> pow_double_double_coerced_directly(2, 2)
    8.0
    >>> x = pow_double_double_coerced_directly(-2, 0.5)
    >>> x == x  # nan
    False
    """
    # Because we're assigning directly to a double assume 'cpow'
    # but warn.
    cdef double c = a**b
    return cfunc_taking_double(a**b) + c

def pow_double_int(double a, int b):
    """
    # a few variations of 'double**int'. In all cases
    # Cython should realise that the result can't be complex
    # and avoid going through the soft complex type
    >>> pow_double_int(5, 2)
    double
    double
    double
    double
    double
    """
    c1 = a**b
    c2 = a**2.0
    c3 = a**-2.0
    c4 = a**5
    c5 = a**-5
    print(cython.typeof(c1))
    print(cython.typeof(c2))
    print(cython.typeof(c3))
    print(cython.typeof(c4))
    print(cython.typeof(c5))

def soft_complex_coerced_to_double(double a, double b):
    """
    >>> soft_complex_coerced_to_double(2, 2)
    4.0
    >>> soft_complex_coerced_to_double(-2, 0.25)
    Traceback (most recent call last):
    ...
    TypeError: Cannot convert 'complex' with non-zero imaginary component to 'double' (this most likely comes from the '**' operator; use 'cython.cpow(True)' to return 'nan' instead of a complex number).
    """
    c = a**b
    assert cython.typeof(c) == "soft double complex"
    cdef double d = c  # will raise if complex
    return d

def soft_complex_coerced_to_complex(double a, double b):
    """
    >>> soft_complex_coerced_to_complex(2, 2)
    (4+0j)
    >>> x = soft_complex_coerced_to_complex(-1, 0.5)
    >>> abs(x.real) < 1e-15
    True
    >>> abs(x.imag - 1) < 1e-15
    True
    """
    # This is always fine, but just check it works
    c = a**b
    assert cython.typeof(c) == "soft double complex"
    cdef double complex d = c
    return d

def soft_complex_type_inference_1(double a, double b, pick):
    """
    >>> soft_complex_type_inference_1(2, 1, False)
    soft double complex 2.0
    >>> soft_complex_type_inference_1(2, 3, True)
    soft double complex 4.0
    """
    # double and soft complex should infer to soft-complex
    if pick:
        c = a**2
    else:
        c = a**b
    print(cython.typeof(c), c)

def soft_complex_type_inference_2(double a, double b, expected):
    """
    >>> soft_complex_type_inference_2(2, 1, 1.0)
    soft double complex
    >>> soft_complex_type_inference_2(2, 3, 7.0)
    soft double complex
    """
    # double and soft complex should infer to soft-complex
    c = a**b
    c -= 1
    print(cython.typeof(c))
    delta = abs(c/expected - 1)
    assert delta < 1e-15, delta

def pow_int_int(int a, int b):
    """
    >>> pow_int_int(2, 2)
    double 4.0
    >>> pow_int_int(2, -2)
    double 0.25
    """
    c = a**b
    print(cython.typeof(c), c)

@cython.cpow(True)
def pow_int_int_cpow(int a, int b):
    """
    >>> pow_int_int_cpow(2, 2)
    int 4
    >>> pow_int_int_cpow(2, -2)
    int 0
    """
    c = a**b
    print(cython.typeof(c), c)

cdef cfunc_taking_int(int x):
    return x

def pow_int_int_coerced_directly(int a, int b):
    """
    Generates two warnings about using cpow.
    The actual behaviour isn't too easy to distinguish
    without inspecting the c code though.
    >>> pow_int_int_coerced_directly(2, 2)
    8
    """
    cdef int c = a**b
    return cfunc_taking_int(a**b) + c

def pow_int_int_non_negative(int a, unsigned int b):
    """
    A couple of combinations of non-negative values for the
    exponent, which lets us fall back to int as a return type
    >>> pow_int_int_non_negative(5, 3)
    unsigned int
    long
    """
    c1 = a**b
    c2 = a**5
    print(cython.typeof(c1))
    print(cython.typeof(c2))


ctypedef double f64

def pythagoras_with_typedef(double a, double b):
    # see https://github.com/cython/cython/issues/5203
    """
    >>> rc = pythagoras_with_typedef(2.0, 2.0)
    >>> pyresult = 1.0 / (2 * 2.0 ** 2) ** 0.5
    >>> pyresult - 0.001 < rc < pyresult + 0.001  or  (rc, pyresult)
    True
    """
    cdef f64 result = a * a + b * b
    result = 1.0 / result ** 0.5
    return result


@cython.cpow(False)
def power_coercion_in_nogil_1(double a, double b):
    """
    >>> power_coercion_in_nogil_1(2., 2.)
    4.0
    >>> power_coercion_in_nogil_1(-1., 0.5)
    Traceback (most recent call last):
    ...
    TypeError: Cannot convert 'complex' with non-zero imaginary component to 'double' (this most likely comes from the '**' operator; use 'cython.cpow(True)' to return 'nan' instead of a complex number).
    """
    cdef double c
    with nogil:
        c = a**b
    return c


cdef double nogil_fun(double x) nogil:
    return x

def power_coercion_in_nogil_2(double a, double b):
    """
    >>> power_coercion_in_nogil_2(2., 2.)
    4.0
    >>> power_coercion_in_nogil_2(-1., 0.5)
    Traceback (most recent call last):
    ...
    TypeError: Cannot convert 'complex' with non-zero imaginary component to 'double' (this most likely comes from the '**' operator; use 'cython.cpow(True)' to return 'nan' instead of a complex number).
    """
    c = a**b
    with nogil:
        d = nogil_fun(c)
    return d


def power_coercion_in_nogil_3(double a, double b, double c):
    """
    >>> power_coercion_in_nogil_3(2., 2., 1.0)
    0.25
    >>> power_coercion_in_nogil_3(-1., 0.5, 1.0)
    Traceback (most recent call last):
    ...
    TypeError: Cannot convert 'complex' with non-zero imaginary component to 'double' (this most likely comes from the '**' operator; use 'cython.cpow(True)' to return 'nan' instead of a complex number).
    """
    with nogil:
        c /= a**b
    return c


_WARNINGS = """
58:21: Treating '**' as if 'cython.cpow(True)' since it is directly assigned to a a non-complex C numeric type. This is likely to be fragile and we recommend setting 'cython.cpow' explicitly.
59:32: Treating '**' as if 'cython.cpow(True)' since it is directly assigned to a a non-complex C numeric type. This is likely to be fragile and we recommend setting 'cython.cpow' explicitly.
174:18: Treating '**' as if 'cython.cpow(True)' since it is directly assigned to a an integer C numeric type. This is likely to be fragile and we recommend setting 'cython.cpow' explicitly.
175:29: Treating '**' as if 'cython.cpow(True)' since it is directly assigned to a an integer C numeric type. This is likely to be fragile and we recommend setting 'cython.cpow' explicitly.
"""



DEF USTUFF = u"Spam"

def uf():
    """
    >>> uf()
    It works!
    """
    IF USTUFF == u"Spam":
        print "It works!"
    ELSE:
        print "Doesn't work"

DEF BSTUFF = b"Spam"

def bf():
    """
    >>> bf()
    It works!
    """
    IF BSTUFF == b"Spam":
        print "It works!"
    ELSE:
        print "Doesn't work"



# coding: utf-8

cdef extern from *:
    const Py_ssize_t PY_SSIZE_T_MIN
    const Py_ssize_t PY_SSIZE_T_MAX

SSIZE_T_MAX = PY_SSIZE_T_MAX
SSIZE_T_MIN = PY_SSIZE_T_MIN


def slice_start_end(unicode s, Py_ssize_t i, Py_ssize_t j):
    """
    >>> slice_start_end('abcdef', 2, 3)
    c
    >>> slice_start_end('abcdef', 2, 10)
    cdef
    >>> slice_start_end('abcdef', 0, 5)
    abcde
    >>> slice_start_end('abcdef', -6, -1)
    abcde
    >>> slice_start_end('abcdef', -6, -7)
    <BLANKLINE>
    >>> slice_start_end('abcdef', -7, -7)
    <BLANKLINE>
    >>> slice_start_end('aАbБcСdДeЕfФ', 2, 8)
    bБcСdД
    >>> slice_start_end('АБСДЕФ', 2, 4)
    СД
    >>> slice_start_end('АБСДЕФ', -4, -2)
    СД
    >>> slice_start_end(None, 2, 4)
    Traceback (most recent call last):
    TypeError: 'NoneType' object is not subscriptable
    >>> slice_start_end('abcdef', SSIZE_T_MAX, SSIZE_T_MIN)
    <BLANKLINE>
    """
    print(s[i:j])


def slice_start(unicode s, Py_ssize_t i, Py_ssize_t j):
    """
    >>> slice_start('abcdef', 2, 3)
    cdef
    >>> slice_start('abcdef', 2, 10)
    cdef
    >>> slice_start('abcdef', 0, 5)
    abcdef
    >>> slice_start('abcdef', -6, -1)
    abcdef
    >>> slice_start('abcdef', -6, -7)
    abcdef
    >>> slice_start('abcdef', -7, -7)
    abcdef
    >>> slice_start('aАbБcСdДeЕfФ', 2, 8)
    bБcСdДeЕfФ
    >>> slice_start('АБСДЕФ', 2, 4)
    СДЕФ
    >>> slice_start('АБСДЕФ', -4, -2)
    СДЕФ
    >>> slice_start(None, 2, 4)
    Traceback (most recent call last):
    TypeError: 'NoneType' object is not subscriptable
    >>> slice_start('abcdef', SSIZE_T_MAX, SSIZE_T_MIN)
    <BLANKLINE>
    """
    print(s[i:])


def slice_end(unicode s, Py_ssize_t i, Py_ssize_t j):
    """
    >>> slice_end('abcdef', 2, 3)
    ab
    >>> slice_end('abcdef', 2, 10)
    ab
    >>> slice_end('abcdef', 0, 5)
    <BLANKLINE>
    >>> slice_end('abcdef', -6, -1)
    <BLANKLINE>
    >>> slice_end('abcdef', -6, -7)
    <BLANKLINE>
    >>> slice_end('abcdef', -7, -7)
    <BLANKLINE>
    >>> slice_end('aАbБcСdДeЕfФ', 2, 8)
    aА
    >>> slice_end('АБСДЕФ', 2, 4)
    АБ
    >>> slice_end('АБСДЕФ', -4, -2)
    АБ
    >>> slice_end(None, 2, 4)
    Traceback (most recent call last):
    TypeError: 'NoneType' object is not subscriptable
    >>> slice_end('abcdef', SSIZE_T_MAX, SSIZE_T_MIN)
    abcdef
    """
    print(s[:i])


def slice_all(unicode s, Py_ssize_t i, Py_ssize_t j):
    """
    >>> slice_all('abcdef', 2, 3)
    abcdef
    >>> slice_all('abcdef', 2, 10)
    abcdef
    >>> slice_all('abcdef', 0, 5)
    abcdef
    >>> slice_all('abcdef', -6, -1)
    abcdef
    >>> slice_all('abcdef', -6, -7)
    abcdef
    >>> slice_all('abcdef', -7, -7)
    abcdef
    >>> slice_all('aАbБcСdДeЕfФ', 2, 8)
    aАbБcСdДeЕfФ
    >>> slice_all('АБСДЕФ', 2, 4)
    АБСДЕФ
    >>> slice_all('АБСДЕФ', -4, -2)
    АБСДЕФ
    >>> slice_all(None, 2, 4)
    Traceback (most recent call last):
    TypeError: 'NoneType' object is not subscriptable
    >>> slice_all('abcdef', SSIZE_T_MAX, SSIZE_T_MIN)
    abcdef
    """
    print(s[:])


def slice_start_none(unicode s, Py_ssize_t i, Py_ssize_t j):
    """
    >>> slice_start_none('abcdef', 2, 3)
    cdef
    >>> slice_start_none('abcdef', 0, 5)
    abcdef
    >>> slice_start_none('abcdef', -6, -1)
    abcdef
    >>> slice_start_none('abcdef', -6, -7)
    abcdef
    >>> slice_start_none('abcdef', -7, -7)
    abcdef
    >>> slice_start_none('aАbБcСdДeЕfФ', 2, 8)
    bБcСdДeЕfФ
    >>> slice_start_none('АБСДЕФ', 2, 4)
    СДЕФ
    >>> slice_start_none('АБСДЕФ', -4, -2)
    СДЕФ
    >>> slice_start_none(None, 2, 4)
    Traceback (most recent call last):
    TypeError: 'NoneType' object is not subscriptable
    >>> slice_start_none('abcdef', SSIZE_T_MAX, SSIZE_T_MIN)
    <BLANKLINE>
    """
    print(s[i:None])


def slice_none_end(unicode s, Py_ssize_t i, Py_ssize_t j):
    """
    >>> slice_none_end('abcdef', 2, 3)
    ab
    >>> slice_none_end('abcdef', 0, 5)
    <BLANKLINE>
    >>> slice_none_end('abcdef', -6, -1)
    <BLANKLINE>
    >>> slice_none_end('abcdef', -6, -7)
    <BLANKLINE>
    >>> slice_none_end('abcdef', -7, -7)
    <BLANKLINE>
    >>> slice_none_end('aАbБcСdДeЕfФ', 2, 8)
    aА
    >>> slice_none_end('АБСДЕФ', 2, 4)
    АБ
    >>> slice_none_end('АБСДЕФ', -4, -2)
    АБ
    >>> slice_none_end(None, 2, 4)
    Traceback (most recent call last):
    TypeError: 'NoneType' object is not subscriptable
    >>> slice_none_end('abcdef', SSIZE_T_MAX, SSIZE_T_MIN)
    abcdef
    """
    print(s[None:i])


def slice_none_none(unicode s, Py_ssize_t i, Py_ssize_t j):
    """
    >>> slice_none_none('abcdef', 2, 3)
    abcdef
    >>> slice_none_none('abcdef', 0, 5)
    abcdef
    >>> slice_none_none('abcdef', -6, -1)
    abcdef
    >>> slice_none_none('abcdef', -6, -7)
    abcdef
    >>> slice_none_none('abcdef', -7, -7)
    abcdef
    >>> slice_none_none('aАbБcСdДeЕfФ', 2, 8)
    aАbБcСdДeЕfФ
    >>> slice_none_none('АБСДЕФ', 2, 4)
    АБСДЕФ
    >>> slice_none_none('АБСДЕФ', -4, -2)
    АБСДЕФ
    >>> slice_none_none(None, 2, 4)
    Traceback (most recent call last):
    TypeError: 'NoneType' object is not subscriptable
    >>> slice_none_none('abcdef', SSIZE_T_MAX, SSIZE_T_MIN)
    abcdef
    """
    print(s[None:None])



# tag: cpp

cimport cython
from cython.operator import dereference as deref

cdef extern from "cpp_templates_helper.h":
    cdef cppclass Wrap[T, AltType=*, UndeclarableAltType=*]:
        Wrap(T)
        void set(T)
        T get()
        bint operator==(Wrap[T])

        AltType get_alt_type()
        void set_alt_type(AltType)

        UndeclarableAltType create()
        bint accept(UndeclarableAltType)

    cdef cppclass Pair[T1,T2]:
        Pair(T1,T2)
        T1 first()
        T2 second()
        bint operator==(Pair[T1,T2])
        bint operator!=(Pair[T1,T2])

    cdef cppclass SuperClass[T1, T2]:
        pass

    cdef cppclass SubClass[T2, T3](SuperClass[T2, T3]):
        pass

    cdef cppclass Div[T]:
        @staticmethod
        T half(T value)

def test_int(int x, int y):
    """
    >>> test_int(3, 4)
    (3, 4, False)
    >>> test_int(100, 100)
    (100, 100, True)
    """
    try:
        a = new Wrap[int](x)
        b = new Wrap[int](0)
        b.set(y)
        return a.get(), b.get(), a[0] == b[0]
    finally:
        del a, b


def test_double(double x, double y):
    """
    >>> test_double(3, 3.5)
    (3.0, 3.5, False)
    >>> test_double(100, 100)
    (100.0, 100.0, True)
    """
    try:
        a = new Wrap[double](x)
        b = new Wrap[double](-1)
        b.set(y)
        return a.get(), b.get(), deref(a) == deref(b)
    finally:
        del a, b


def test_default_template_arguments(double x):
    """
    >>> test_default_template_arguments(3.5)
    (3.5, 3.0)
    """
    try:
        a = new Wrap[double](x)
        b = new Wrap[double, int, long](x)

        ax = a.get_alt_type()
        a.set_alt_type(ax)
        assert a.accept(a.create())  # never declared

        bx = b.get_alt_type()
        b.set_alt_type(bx)

        bc = b.create()              # declaration here is fine
        assert b.accept(bc)

        return a.get(), b.get()
    finally:
        del a


def test_pair(int i, double x):
    """
    >>> test_pair(1, 1.5)
    (1, 1.5, True, False)
    >>> test_pair(2, 2.25)
    (2, 2.25, True, False)
    """
    try:
        pair = new Pair[int, double](i, x)
        return pair.first(), pair.second(), deref(pair) == deref(pair), deref(pair) != deref(pair)
    finally:
        del pair

def test_ptr(int i):
    """
    >>> test_ptr(3)
    3
    >>> test_ptr(5)
    5
    """
    try:
        w = new Wrap[int*](&i)
        return deref(w.get())
    finally:
        del w

cdef double f(double x):
    return x*x

def test_func_ptr(double x):
    """
    >>> test_func_ptr(3)
    9.0
    >>> test_func_ptr(-1.5)
    2.25
    """
    try:
        w = new Wrap[double (*)(double)](&f)
        return w.get()(x)
    finally:
        del w

def test_typeof(double x):
    """
    >>> test_func_ptr(3)
    9.0
    >>> test_func_ptr(-1.5)
    2.25
    """
    try:
        w = new Wrap[cython.typeof(&f)](&f)
        return w.get()(x)
    finally:
        del w

def test_cast_template_pointer():
    """
    >>> test_cast_template_pointer()
    """
    cdef SubClass[int, float] *sub = new SubClass[int, float]()
    cdef SuperClass[int, float] *sup

    sup = sub
    sup = <SubClass[int, float] *> sub

def test_static(x):
    """
    >>> test_static(2)
    (1, 1.0)
    >>> test_static(3)
    (1, 1.5)
    """
    return Div[int].half(x), Div[double].half(x)

def test_pure_syntax(int i):
    """
    >>> test_ptr(3)
    3
    >>> test_ptr(5)
    5
    """
    try:
        w = new Wrap[cython.pointer(int)](&i)
        return deref(w.get())
    finally:
        del w



# mode: run
# tag: exttype, tpnew

from __future__ import print_function

from cpython.object cimport PyTypeObject


cdef gobble(a, b):
    print(a, b)


def tp_new_ptr(exttype):
    assert isinstance(exttype, type)
    tp = <PyTypeObject*> exttype
    return <unsigned long long><void*>tp.tp_new


cdef class Empty:
    """
    >>> n = Empty()
    >>> isinstance(n, Empty)
    True
    >>> tp_new_ptr(Empty) != 0
    True
    """


cdef class EmptySubclass(Empty):
    """
    >>> n = EmptySubclass()
    >>> isinstance(n, EmptySubclass)
    True
    >>> tp_new_ptr(EmptySubclass) != 0
    True
    >>> tp_new_ptr(EmptySubclass) == tp_new_ptr(Empty)
    True
    """


cdef class CInit:
    """
    >>> c = CInit()
    >>> isinstance(c, CInit)
    True
    """
    def __cinit__(self):
        assert self is not None


cdef class Spam:
    """
    >>> s = Spam(12)
    >>> s.eat()
    12 42
    """
    cdef eggs
    cdef int ham

    def __cinit__(self, eggs):
        self.eggs = eggs
        self.ham = 42

    def __dealloc__(self):
        self.ham = 0

    def eat(self):
        gobble(self.eggs, self.ham)


def f(Spam spam):
    """
    >>> s = Spam(12)
    >>> f(s)   # doctest: +ELLIPSIS
    Traceback (most recent call last):
    AttributeError: '...Spam' object has no attribute 'foo'
    >>> s.eat()
    12 42
    >>> class Spam2(Spam):
    ...     foo = 1
    >>> s = Spam2(12)
    >>> s.eat()
    12 42
    >>> f(s)
    >>> s.eat()
    12 42
    """
    x = spam.eggs
    y = spam.ham
    z = spam.foo
    spam.eggs = x
    spam.ham = y
    spam.foo = z



# ticket: t276
# mode: compile

__doc__ = u"""
"""

cdef class A:
    cdef __weakref__

ctypedef public class B [type B_Type, object BObject]:
    cdef __weakref__

cdef public class C [type C_Type, object CObject]:
    cdef __weakref__




# mode: error

cimport cython

ctypedef int USERTYPE

# Functions

@cython.callspec("")
cdef void h1(): pass

@cython.callspec("__cdecl")
cdef void __cdecl h2(): pass

@cython.callspec("__stdcall")
cdef void __stdcall h3(): pass

@cython.callspec("__fastcall")
cdef void __fastcall h4(): pass

cdef USERTYPE h5(): return 0

cdef USERTYPE __cdecl h6(): return 0

cdef USERTYPE __stdcall h7(): return 0

cdef USERTYPE __fastcall h8(): return 0

@cython.callspec("__cdecl")
cdef void __stdcall herr1(): pass # fail

@cython.callspec("__cdecl")
cdef void __fastcall herr2(): pass # fail

# Pointer typedefs

ctypedef void (*PT1)()
ctypedef void (__cdecl *PT2)()
ctypedef void (__stdcall *PT3)()
ctypedef void (__fastcall *PT4)()
ctypedef USERTYPE (*PT5)()
ctypedef USERTYPE (__cdecl *PT6)()
ctypedef USERTYPE (__stdcall *PT7)()
ctypedef USERTYPE (__fastcall *PT8)()

# Pointers

cdef void (*p1)()
cdef void (__cdecl *p2)()
cdef void (__stdcall *p3)()
cdef void (__fastcall *p4)()
cdef USERTYPE (*p5)()
cdef USERTYPE (__cdecl *p6)()
cdef USERTYPE (__stdcall *p7)()
cdef USERTYPE (__fastcall *p8)()

cdef PT1 pt1
cdef PT2 pt2
cdef PT3 pt3
cdef PT4 pt4
cdef PT5 pt5
cdef PT6 pt6
cdef PT7 pt7
cdef PT8 pt8

# Assignments

p1 = pt1 = p2 = pt2 = h1
p1 = pt1 = p2 = pt2 = h2
p3 = pt3 = h3
p4 = pt4 = h4

p5 = pt5 = p6 = pt6 = h5
p5 = pt5 = p6 = pt6 = h6
p7 = pt7 = h7
p8 = pt8 = h8

#p1 = h2 # fail
#p1 = h3 # fail
#p1 = h4 # fail

#p2 = h1 # fail
#p2 = h3 # fail
#p2 = h4 # fail

_ERRORS = u"""
30:25: cannot have both '__stdcall' and '__cdecl' calling conventions
33:26: cannot have both '__fastcall' and '__cdecl' calling conventions
"""
#31:14: Cannot assign type 'void (__cdecl )(void)' to 'void (*)(void)'
#32:14: Cannot assign type 'void (__stdcall )(void)' to 'void (*)(void)'
#33:14: Cannot assign type 'void (__fastcall )(void)' to 'void (*)(void)'
#35:14: Cannot assign type 'void (void)' to 'void (__cdecl *)(void)'
#36:14: Cannot assign type 'void (__stdcall )(void)' to 'void (__cdecl *)(void)'
#37:14: Cannot assign type 'void (__fastcall )(void)' to 'void (__cdecl *)(void)'



# mode: run
# tag: pure3.0

import cython


def empty_float():
    """
    >>> float()
    0.0
    >>> empty_float()
    0.0
    """
    x = float()
    return x


def float_conjugate():
    """
    >>> float_call_conjugate()
    1.5
    """
    x = 1.5 .conjugate()
    return x


def float_call_conjugate():
    """
    >>> float_call_conjugate()
    1.5
    """
    x = float(1.5).conjugate()
    return x


def from_int(i):
    """
    >>> from_int(0)
    0.0
    >>> from_int(1)
    1.0
    >>> from_int(-1)
    -1.0
    >>> from_int(99)
    99.0
    >>> from_int(-99)
    -99.0

    >>> for exp in (14, 15, 16, 30, 31, 32, 52, 53, 54, 60, 61, 62, 63, 64):
    ...     for sign in (1, 0, -1):
    ...         value = (sign or 1) * 2**exp + sign
    ...         float_value = from_int(value)
    ...         assert float_value == float(value), "expected %s2**%s+%s == %r, got %r, difference %r" % (
    ...             '-' if sign < 0 else '', exp, sign, float(value), float_value, float_value - float(value))
    """
    return float(i)


@cython.test_assert_path_exists(
    "//CoerceToPyTypeNode",
    "//CoerceToPyTypeNode//PythonCapiCallNode",
)
def from_bytes(s: bytes):
    """
    >>> from_bytes(b"123")
    123.0
    >>> from_bytes(b"123.25")
    123.25
    >>> from_bytes(b"98_5_6.2_1")
    9856.21
    >>> from_bytes(b"12_4_131_123123_1893798127398123_19238712_128937198237.8222113_519879812387")
    1.2413112312318938e+47
    >>> from_bytes(b"123E100")
    1.23e+102
    >>> from_bytes(b"12__._3")  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ...12__._3...
    >>> from_bytes(b"_12.3")  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ..._12.3...
    >>> from_bytes(b"12.3_")  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ...12.3_...
    >>> from_bytes(b"na_n")  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ...na_n...
    >>> from_bytes(b"_" * 10000)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ...____...
    >>> from_bytes(None)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError...
    """
    return float(s)


@cython.test_assert_path_exists(
    "//CoerceToPyTypeNode",
    "//CoerceToPyTypeNode//PythonCapiCallNode",
)
def from_bytes_literals():
    """
    >>> from_bytes_literals()
    (123.0, 123.23, 123.76, 1e+100)
    """
    return float(b"123"), float(b"123.23"), float(b"12_3.7_6"), float(b"1e100")


@cython.test_assert_path_exists(
    "//CoerceToPyTypeNode",
    "//CoerceToPyTypeNode//PythonCapiCallNode",
)
def from_bytearray(s: bytearray):
    """
    >>> from_bytearray(bytearray(b"123"))
    123.0
    >>> from_bytearray(bytearray(b"123.25"))
    123.25
    >>> from_bytearray(bytearray(b"98_5_6.2_1"))
    9856.21
    >>> from_bytearray(bytearray(b"12_4_131_123123_1893798127398123_19238712_128937198237.8222113_519879812387"))
    1.2413112312318938e+47
    >>> from_bytearray(bytearray(b"123E100"))
    1.23e+102
    >>> from_bytearray(bytearray(b"12__._3"))  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ...12__._3...
    >>> from_bytearray(bytearray(b"_12.3"))  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ..._12.3...
    >>> from_bytearray(bytearray(b"12.3_"))  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ...12.3_...
    >>> from_bytearray(bytearray(b"in_f"))  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ...in_f...
    >>> from_bytearray(None)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError...
    """
    return float(s)


@cython.test_assert_path_exists(
    "//CoerceToPyTypeNode",
    "//CoerceToPyTypeNode//PythonCapiCallNode",
)
def from_str(s: 'str'):
    """
    >>> from_str("123")
    123.0
    >>> from_str("123.25")
    123.25
    >>> from_str("3_21.2_5")
    321.25
    >>> from_str("12_4_131_123123_1893798127398123_19238712_128937198237.8222113_519879812387")
    1.2413112312318938e+47
    >>> from_str("123E100")
    1.23e+102
    >>> from_str("12__._3")  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ...12__._3...
    >>> from_str("_12.3")  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ..._12.3...
    >>> from_str("12.3_")  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ...12.3_...
    >>> from_str("n_an")  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ...n_an...
    >>> from_str(None)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError...
    """
    return float(s)


@cython.test_assert_path_exists(
    "//CoerceToPyTypeNode",
    "//CoerceToPyTypeNode//PythonCapiCallNode",
)
def from_str_literals():
    """
    >>> from_str_literals()
    (123.0, 123.23, 124.23, 1e+100)
    """
    return float("123"), float("123.23"), float("1_2_4.2_3"), float("1e100")


@cython.test_assert_path_exists(
    "//CoerceToPyTypeNode",
    "//CoerceToPyTypeNode//PythonCapiCallNode",
)
def from_unicode(s: 'unicode'):
    """
    >>> from_unicode(u"123")
    123.0
    >>> from_unicode(u"123.25")
    123.25
    >>> from_unicode(u"12_4.8_5")
    124.85
    >>> from_unicode(u"12_4_131_123123_1893798127398123_19238712_128937198237.8222113_519879812387")
    1.2413112312318938e+47
    >>> from_unicode(u"123E100")
    1.23e+102
    >>> from_unicode("೬")
    6.0
    >>> from_unicode(u"123.23\\N{PUNCTUATION SPACE}")
    123.23
    >>> from_unicode(u"\\N{PUNCTUATION SPACE} 123.23 \\N{PUNCTUATION SPACE}")
    123.23
    >>> from_unicode(u"\\N{PUNCTUATION SPACE} 12_3.2_3 \\N{PUNCTUATION SPACE}")
    123.23
    >>> from_unicode(u"\\N{PUNCTUATION SPACE} " * 25 + u"123.54 " + u"\\N{PUNCTUATION SPACE} " * 22)  # >= 40 chars
    123.54
    >>> from_unicode(u"\\N{PUNCTUATION SPACE} " * 25 + u"1_23.5_4 " + u"\\N{PUNCTUATION SPACE} " * 22)
    123.54
    >>> from_unicode(u"\\N{PUNCTUATION SPACE} " + u"123.54 " * 2 + u"\\N{PUNCTUATION SPACE}")  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ...123.54 123.54...
    >>> from_unicode(u"\\N{PUNCTUATION SPACE} " * 25 + u"123.54 " * 2 + u"\\N{PUNCTUATION SPACE} " * 22)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ...123.54 123.54...
    >>> from_unicode(u"_12__._3")  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ..._12__._3...
    >>> from_unicode(u"_12.3")  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ..._12.3...
    >>> from_unicode(u"12.3_")  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ...12.3_...
    >>> from_unicode(u"i_nf")  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    ValueError: ...i_nf...
    >>> from_unicode(None)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
    TypeError...
    """
    return float(s)


@cython.test_assert_path_exists(
    "//CoerceToPyTypeNode",
    "//CoerceToPyTypeNode//PythonCapiCallNode",
)
def from_unicode_literals():
    """
    >>> from_unicode_literals()
    (123.0, 123.23, 123.45, 1e+100, 123.23)
    """
    return float(u"123"), float(u"123.23"), float(u"12_3.4_5"), float(u"1e100"), float(u"123.23\N{PUNCTUATION SPACE}")


def catch_valueerror(val):
    """
    >>> catch_valueerror("foo")
    False
    >>> catch_valueerror(u"foo")
    False
    >>> catch_valueerror(b"foo")
    False
    >>> catch_valueerror(bytearray(b"foo"))
    False
    >>> catch_valueerror("-1")
    -1.0
    """
    try:
        return float(val)
    except ValueError:
        return False



# ticket: t158
# mode: error

def mult_decl_test():
    print "%s" % vv
    print "%s" % s
    cdef str s, vv = "Test"

def def_test():
    cdef int j = 10
    i[0] = j
    cdef int *i = NULL # pointer variables are special case

cdef cdef_test():
    cdef int j = 10
    i[0] = j
    print "%d" % i[0]
    cdef int *i = NULL

cpdef cpdef_test():
    cdef int j = 10
    i[0] = j
    print "%d" % i[0]
    cdef int *i = NULL

s.upper()
cdef str s = "Test"

class Foo(object):
    def bar(self, x, y):
        cdef unsigned long w = 20
        z = w + t
        cdef int t = 10

cdef class Foo2(object):
    print '%s' % r # check error inside class scope
    cdef str r
    def bar(self, x, y):
        cdef unsigned long w = 20
        self.r = c'r'
        print self.r
        z = w + g(t)
        cdef int t = 10

def g(x):
    return x

cdef int d = 20
baz[0] = d
cdef int *baz

print var[0][0]
cdef unsigned long long[100][100] var

_ERRORS = u"""
5:17: local variable 'vv' referenced before assignment
6:17: local variable 's' referenced before assignment
7:13: cdef variable 's' declared after it is used
7:16: cdef variable 'vv' declared after it is used
12:14: cdef variable 'i' declared after it is used
18:14: cdef variable 'i' declared after it is used
24:14: cdef variable 'i' declared after it is used
27:9: cdef variable 's' declared after it is used
33:17: cdef variable 't' declared after it is used
43:17: cdef variable 't' declared after it is used
50:10: cdef variable 'baz' declared after it is used
53:34: cdef variable 'var' declared after it is used
"""
# FIXME not detected
#37:13: cdef variable 'r' declared after it is used



# mode: run
# tag: cpp, cpp17, no-cpp-locals, openmp
# no-cpp-locals because the test is already run with it explicitly set

# cython: cpp_locals=True

from cython.parallel cimport prange

cdef extern from *:
    """
    class Test {
    public:
        Test() = delete;
        Test(int v) : value(v) {}

        int get_value() const { return value; }
    private:
        int value;
    };
    """
    cdef cppclass Test:
        Test(int) nogil
        int get_value()

def test():
    """
    >>> test()
    9
    """
    cdef int i
    for i in prange(10, nogil=True):
        var = Test(i)
    print(var.get_value())



def f(x):
    """
    >>> f(1)
    (1, 17)
    """
    cdef int y
    z = 42
    with nogil:
        y = 17
    z = x
    return z,y

def g():
    """
    >>> g()
    1
    """
    with nogil:
        h()
    return 1

cdef int h() except -1 nogil:
    pass



cimport cython

cdef object two = 2

cdef int size_in_bits_ = sizeof(INT) * 8
cdef bint is_signed_ = not ((<INT>-1) > 0)
cdef INT max_value_ = <INT>(two ** (size_in_bits_ - is_signed_) - 1)
cdef INT min_value_ = ~max_value_
cdef INT half_ = max_value_ // <INT>2

# Python visible.
size_in_bits = size_in_bits_
is_signed = is_signed_
max_value = max_value_
min_value = min_value_
half = half_


import operator
from libc.math cimport sqrt

cpdef check(func, op, a, b):
    cdef INT res = 0, op_res = 0
    cdef bint func_overflow = False
    cdef bint assign_overflow = False
    try:
        res = func(a, b)
    except OverflowError:
        func_overflow = True
    try:
        op_res = op(a, b)
    except OverflowError:
        assign_overflow = True
    assert func_overflow == assign_overflow, "Inconsistent overflow: %s(%s, %s)" % (func, a, b)
    if not func_overflow:
        assert res == op_res, "Inconsistent values: %s(%s, %s) == %s != %s" % (func, a, b, res, op_res)

medium_values = (max_value_ / 2, max_value_ / 3, min_value_ / 2, <INT>sqrt(<long double>max_value_) - <INT>1, <INT>sqrt(<long double>max_value_) + 1)
def run_test(func, op):
    cdef INT offset, b
    check(func, op, 300, 200)
    check(func, op, max_value_, max_value_)
    check(func, op, max_value_, min_value_)
    if not is_signed_ or not func is test_sub:
        check(func, op, min_value_, min_value_)

    for offset in range(5):
        check(func, op, max_value_ - <INT>1, offset)
        check(func, op, min_value_ + <INT>1, offset)
        if is_signed_:
            check(func, op, max_value_ - 1, 2 - offset)
            check(func, op, min_value_ + 1, 2 - offset)

    for offset in range(9):
        check(func, op, max_value_ / <INT>2, offset)
        check(func, op, min_value_ / <INT>3, offset)
        check(func, op, max_value_ / <INT>4, offset)
        check(func, op, min_value_ / <INT>5, offset)
        if is_signed_:
            check(func, op, max_value_ / 2, 4 - offset)
            check(func, op, min_value_ / 3, 4 - offset)
            check(func, op, max_value_ / -4, 3 - offset)
            check(func, op, min_value_ / -5, 3 - offset)

    for offset in range(-3, 4):
        for a in medium_values:
            for b in medium_values:
                check(func, op, a, b + offset)

@cython.overflowcheck(True)
def test_add(INT a, INT b):
    """
    >>> test_add(1, 2)
    3
    >>> test_add(max_value, max_value)   #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ...
    OverflowError: value too large
    >>> run_test(test_add, operator.add)
    """
    return int(a + b)
    
@cython.overflowcheck(True)
def test_sub(INT a, INT b):
    """
    >>> test_sub(10, 1)
    9
    >>> test_sub(min_value, 1)   #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ...
    OverflowError: value too large
    >>> run_test(test_sub, operator.sub)
    """
    return int(a - b)

@cython.overflowcheck(True)
def test_mul(INT a, INT b):
    """
    >>> test_mul(11, 13)
    143
    >>> test_mul(max_value / 2, max_value / 2)   #doctest: +ELLIPSIS
    Traceback (most recent call last):
    ...
    OverflowError: value too large
    >>> run_test(test_mul, operator.mul)
    """
    return int(a * b)

@cython.overflowcheck(True)
def test_nested_add(INT a, INT b, INT c):
    """
    >>> test_nested_add(1, 2, 3)
    6
    >>> expect_overflow(test_nested_add, half + 1, half + 1, half + 1)
    >>> expect_overflow(test_nested_add, half - 1, half - 1, half - 1)
    """
    return int(a + b + c)

def expect_overflow(func, *args):
    try:
        res = func(*args)
    except OverflowError:
        return
    assert False, "Expected OverflowError, got %s" % res

cpdef format(INT value):
    """
    >>> format(1)
    '1'
    >>> format(half - 1)
    'half - 1'
    >>> format(half)
    'half'
    >>> format(half + 2)
    'half + 2'
    >>> format(half + half - 3)
    'half + half - 3'
    >>> format(max_value)
    'max_value'
    """
    if value == max_value_:
        return "max_value"
    elif value == half_:
        return "half"
    elif max_value_ - value <= max_value_ // <INT>4:
        return "half + half - %s" % (half_ + half_ - value)
    elif max_value_ - value <= half_:
        return "half + %s" % (value - half_)
    elif max_value_ - value <= half_ + max_value_ // <INT>4:
        return "half - %s" % (half_ - value)
    else:
        return "%s" % value

cdef INT called(INT value):
    print("called(%s)" % format(value))
    return value

@cython.overflowcheck(True)
def test_nested(INT a, INT b, INT c, INT d):
    """
    >>> test_nested_func(1, 2, 3)
    called(5)
    6
    >>> expect_overflow(test_nested, half, half, 1, 1)
    >>> expect_overflow(test_nested, half, 1, half, half)
    >>> expect_overflow(test_nested, half, 2, half, 2)

    >>> print(format(test_nested(half, 2, 0, 1)))
    half + half - 0
    >>> print(format(test_nested(1, 0, half, 2)))
    half + half - 0
    >>> print(format(test_nested(half, 1, 1, half)))
    half + half - 0
    """
    return int(a * b + c * d)

@cython.overflowcheck(True)
def test_nested_func(INT a, INT b, INT c):
    """
    >>> test_nested_func(1, 2, 3)
    called(5)
    6
    >>> expect_overflow(test_nested_func, half + 1, half + 1, half + 1)
    >>> expect_overflow(test_nested_func, half - 1, half - 1, half - 1)
    called(half + half - 2)
    >>> print(format(test_nested_func(1, half - 1, half - 1)))
    called(half + half - 2)
    half + half - 1
    """
    return int(a + called(b + c))


@cython.overflowcheck(True)
def test_add_const(INT a):
    """
    >>> test_add_const(1)
    101
    >>> expect_overflow(test_add_const, max_value)
    >>> expect_overflow(test_add_const , max_value - 99)
    >>> test_add_const(max_value - 100) == max_value
    True
    """
    return int(a + <INT>100)

@cython.overflowcheck(True)
def test_sub_const(INT a):
    """
    >>> test_sub_const(101)
    1
    >>> expect_overflow(test_sub_const, min_value)
    >>> expect_overflow(test_sub_const, min_value + 99)
    >>> test_sub_const(min_value + 100) == min_value
    True
    """
    return int(a - <INT>100)

@cython.overflowcheck(True)
def test_mul_const(INT a):
    """
    >>> test_mul_const(2)
    200
    >>> expect_overflow(test_mul_const, max_value)
    >>> expect_overflow(test_mul_const, max_value // 99)
    >>> test_mul_const(max_value // 100) == max_value - max_value % 100
    True
    """
    return int(a * <INT>100)

@cython.overflowcheck(True)
def test_lshift(INT a, int b):
    """
    >>> test_lshift(1, 10)
    1024
    >>> test_lshift(1, size_in_bits - 2) == 1 << (size_in_bits - 2)
    True
    >>> test_lshift(0, size_in_bits - 1)
    0
    >>> test_lshift(1, size_in_bits - 1) == 1 << (size_in_bits - 1) if not is_signed else True
    True
    >>> if is_signed: expect_overflow(test_lshift, 1, size_in_bits - 1)
    >>> expect_overflow(test_lshift, 0, size_in_bits)
    >>> expect_overflow(test_lshift, 1, size_in_bits)
    >>> expect_overflow(test_lshift, 0, size_in_bits + 1)
    >>> expect_overflow(test_lshift, 1, size_in_bits + 1)
    >>> expect_overflow(test_lshift, 1, 100)
    >>> expect_overflow(test_lshift, max_value, 1)
    >>> test_lshift(max_value, 0) == max_value
    True
    
    >>> check(test_lshift, operator.lshift, 10, 15)
    >>> check(test_lshift, operator.lshift, 10, 30)
    >>> check(test_lshift, operator.lshift, 100, 60)
    """
    return int(a << b)



# mode: run
# cython: language_level=3

# This file tests that methodmangling is applied correctly to
# pure Python decorated classes.

import cython

if cython.compiled:
    # don't run in Python mode since a significant number of the tests
    # are only for Cython features

    def declare(**kwargs):
        return kwargs['__x']

    class RegularClass:
        @cython.locals(__x=cython.int)
        def f1(self, __x, dummy=None):
            """
            Is the locals decorator correctly applied
            >>> c = RegularClass()
            >>> c.f1(1)
            1
            >>> c.f1("a")
            Traceback (most recent call last):
            ...
            TypeError: an integer is required
            >>> c.f1(_RegularClass__x = 1)
            1
            """
            return __x

        def f2(self, x):
            """
            Is the locals decorator correctly applied
            >>> c = RegularClass()
            >>> c.f2(1)
            1
            >>> c.f2("a")
            Traceback (most recent call last):
            ...
            TypeError: an integer is required
            """
            __x = cython.declare(cython.int, x)

            return __x

        def f3(self, x):
            """
            Is the locals decorator correctly applied
            >>> c = RegularClass()
            >>> c.f3(1)
            1
            >>> c.f3("a")
            Traceback (most recent call last):
            ...
            TypeError: an integer is required
            """
            cython.declare(__x=cython.int)
            __x = x

            return __x

        def f4(self, x):
            """
            We shouldn't be tripped up by a function called
            "declare" that is nothing to do with cython
            >>> RegularClass().f4(1)
            1
            """
            return declare(__x=x)
else:
    __doc__ = """
    >>> True
    True
    """  # stops Python2 from failing



# ticket: t359

cdef unsigned char* some_c_unstring = 'test toast taste'

def test_uchar_conversion():
    """
    >>> py_string1, py_string2, py_string3 = test_uchar_conversion()
    >>> print(py_string1.decode('iso8859-1'))
    test toast taste
    >>> print(py_string2.decode('iso8859-1'))
    test toast taste
    >>> print(py_string3.decode('iso8859-1'))
    test toast taste
    """

    cdef object py_string1 = some_c_unstring

    cdef unsigned char* c_unstring_from_py = py_string1
    cdef object py_string2 = c_unstring_from_py

    cdef char* c_string_from_py = py_string2
    cdef object py_string3 = c_string_from_py

    return py_string1, py_string2, py_string3



# mode: run
# tag: pep484, warnings, pure3.6
# ticket: 5643
# cython: language_level=3

try:
    from typing import Optional
except ImportError:
    pass


# no crash
def gh5643_optional(a: Optional[int] = None):
    """
    >>> gh5643_optional()
    True
    >>> gh5643_optional(1)
    False
    """
    return a is None


# no crash
def gh5643_int_untyped(a: int = 1, b = None):
    """
    >>> gh5643_int_untyped(2)
    (False, True)
    >>> gh5643_int_untyped(2, None)
    (False, True)
    >>> gh5643_int_untyped(1, 3)
    (True, False)
    """
    return a == 1, b is None


# used to crash
def gh5643_int_int_none(a: int = 1, b: int = None):  # should warn about missing "Optional[]"
    """
    >>> gh5643_int_int_none()
    (True, True)
    >>> gh5643_int_int_none(2, 3)
    (False, False)
    """
    return a == 1, b is None


def gh5643_int_int_integer(a: int = 1, b: int = 3):
    """
    >>> gh5643_int_int_integer()
    (True, True)
    >>> gh5643_int_int_integer(2, 3)
    (False, True)
    """
    return a == 1, b == 3


# used to crash
def gh5643_int_optional_none(a: int = 1, b: Optional[int] = None):
    """
    >>> gh5643_int_optional_none()
    (True, True)
    >>> gh5643_int_optional_none(2)
    (False, True)
    >>> gh5643_int_optional_none(2, 3)
    (False, False)
    """
    return a == 1, b is None


def gh5643_int_optional_integer(a: int = 1, b: Optional[int] = 2):
    """
    >>> gh5643_int_optional_integer()
    (True, True)
    >>> gh5643_int_optional_integer(2)
    (False, True)
    >>> gh5643_int_optional_integer(2, 3)
    (False, False)
    >>> gh5643_int_optional_integer(2, 2)
    (False, True)
    """
    return a == 1, b == 2


_WARNINGS = """
37:36: PEP-484 recommends 'typing.Optional[...]' for arguments that can be None.
"""




def call_iter1(x):
    """
    >>> [ i for i in iter([1,2,3]) ]
    [1, 2, 3]
    >>> [ i for i in call_iter1([1,2,3]) ]
    [1, 2, 3]
    """
    return iter(x)

class Ints(object):
    def __init__(self):
        self.i = 0
    def __call__(self):
        self.i += 1
        if self.i > 10:
            raise ValueError
        return self.i

def call_iter2(x, sentinel):
    """
    >>> [ i for i in iter(Ints(), 3) ]
    [1, 2]
    >>> [ i for i in call_iter2(Ints(), 3) ]
    [1, 2]
    """
    return iter(x, sentinel)



def sum_array(view: cython.int[:]):
    """
    >>> from array import array
    >>> sum_array( array('i', [1,2,3]) )
    6
    """
    total: cython.int = 0
    for i in range(view.shape[0]):
        total += view[i]
    return total



def test_str_subclass_kwargs(k=None):
    """
    Test passing keywords with names that are not of type ``str``
    but a subclass:

    >>> class StrSubclass(str):
    ...     pass
    >>> class StrNoCompare(str):
    ...     def __eq__(self, other):
    ...         raise RuntimeError("do not compare me")
    ...     def __hash__(self):
    ...         return hash(str(self))
    >>> kwargs = {StrSubclass('k'): 'value'}
    >>> test_str_subclass_kwargs(**kwargs)
    'value'
    >>> kwargs = {StrNoCompare('k'): 'value'}
    >>> test_str_subclass_kwargs(**kwargs)
    Traceback (most recent call last):
    RuntimeError: do not compare me
    """
    return k



"""
Report benchmark results from CSV files in Markdown format.
"""

import csv
import itertools
import operator


def unbreak(s):
    return s.replace(' ', '\N{NO-BREAK SPACE}')


def concat_files(csv_files):
    for csv_file in csv_files:
        with open(csv_file) as f:
            yield from f


def read_rows(csv_rows):
    # CSV Format: benchmark, revision_name, pyversion, tmin, tmed, tmax
    reader = csv.reader(csv_rows)

    # Sort by benchmark name.
    rows = sorted(reader, key=operator.itemgetter(0))
    return rows


def build_table(rows):
    # Collect all revision names and Python versions, keeping their original order.
    # (The set may not be the same for all benchmarks.)
    revisions = list({row[1]: 1 for row in rows})
    python_versions = list({row[2]: 1 for row in rows})

    # Prepare table column mapping and header.
    pos = itertools.count(1)
    column_map = {
        (pyversion, revision):  next(pos)
        for pyversion in python_versions
        for revision in revisions
    }
    header = ["Benchmark timings"] + [f"Py{pyversion}: {revision[:20]}" for (pyversion, revision) in column_map]
    row_template = [''] * len(header)

    # For each benchmark, report all timings in separate columns.
    table = []
    empty_column_indices = set(column_map.values())
    for benchmark, bm_rows in itertools.groupby(rows, key=operator.itemgetter(0)):
        row = row_template[:]
        table.append(row)

        row[0] = benchmark
        for _, revision_name, pyversion, tmin, tmed, tmax, diff in bm_rows:
            column_index = column_map[(pyversion, revision_name)]
            empty_column_indices.discard(column_index)
            row[column_index] = f"{unbreak(tmed)} ({unbreak(diff.strip(' ()'))})" if diff else unbreak(tmed)

    # Strip empty columns, highest to lowest.
    for column_index in sorted(empty_column_indices, reverse=True):
        del header[column_index]
        for row in table:
            del row[column_index]

    return header, table


def generate_markdown(header, table):
    # Size the table columns.
    column_lengths = [
        max(map(len, map(operator.itemgetter(i), itertools.chain([header], table))))
        for i in range(len(header))
    ]

    # Generate Markdown formatted table lines.
    row_format = ("| {:<%ds}" + " | {:>%ds}" * (len(column_lengths) - 1) + " |\n") % tuple(column_lengths)
    format_row = row_format.format

    yield format_row(*header)
    yield format_row(*['-' * length for length in column_lengths])
    yield from itertools.starmap(format_row, table)


def main(csv_files):
    rows = read_rows(concat_files(csv_files))
    header, table = build_table(rows)
    for line in generate_markdown(header, table):
        print(line, end='')


if __name__ == "__main__":
    import sys
    main(sys.argv[1:])



# mode: run
# tag: pycomplex

from cpython.complex cimport complex


def complex_attributes():
    """
    >>> complex_attributes()
    (1.0, 2.0)
    """
    cdef complex c = 1+2j
    return (c.real, c.imag)


def complex_attributes_assign():
    """
    >>> complex_attributes_assign()
    (10.0, 20.0)
    """
    cdef complex c = 1+2j
    c.cval.real, c.cval.imag = 10, 20
    return (c.real, c.imag)


def complex_cstruct_assign():
    """
    >>> complex_cstruct_assign()
    (10.0, 20.0)
    """
    cdef complex c = 1+2j
    cval = &c.cval
    cval.real, cval.imag = 10, 20
    return (c.real, c.imag)


def complex_coercion():
    """
    >>> complex_coercion()
    (1.0, 2.0, 1.0, 2.0)
    """
    cdef complex py_c = 1+2j
    cdef double complex c_c = py_c
    cdef object py = c_c
    return (c_c.real, c_c.imag, py.real, py.imag)


def complex_arg(complex c):
    """
    >>> complex_arg(1+2j)
    (1.0, 2.0)
    """
    return (c.real, c.imag)


def complex_conjugate_nonsimple_float():
    """
    >>> complex_conjugate_nonsimple_float()
    1.0
    """
    x = float(1.0).conjugate()
    return x


cdef double float_result():
    return 1.0

def complex_conjugate_nonsimple():
    """
    >>> complex_conjugate_nonsimple()
    1.0
    """
    x = float_result().conjugate()
    return x


def complex_builtin_function(a, b):
    """
    >>> two = complex_builtin_function(1.0, 0.5)
    >>> two == (2+0j)  or  two
    True
    """
    return complex(a) / complex(b)



# mode: run

cimport cython

@cython.final
cdef class AClass:
    cdef c_do_something(self, cython.integral x):
        return cython.typeof(x), x

    def do_something(self, x, selector):
        """
        >>> i = AClass()
        >>> i.do_something(1, True)
        ('short', 1)
        >>> i.do_something(2, False)
        ('long', 2)
        """
        if selector:
            return self.c_do_something[short](x)
        else:
            return self.c_do_something[long](x)



# mode: run

from __future__ import print_function

import cython

# https://github.com/cython/cython/issues/3954
# calls to the __class__ attributes of builtin types were optimized to something invalid

@cython.locals(d=dict)
def test_dict(d):
    """
    >>> test_dict({})
    dict
    {}
    """
    print(d.__class__.__name__)
    print(d.__class__())

@cython.locals(i=int)
def test_int(i):
    """
    >>> test_int(0)
    int
    0
    """
    print(i.__class__.__name__)
    print(i.__class__())

@cython.cclass
class C:
    def __str__(self):
        return "I'm a C object"

@cython.locals(c=C)
def test_cdef_class(c):
    """
    # This wasn't actually broken but is worth testing anyway
    >>> test_cdef_class(C())
    C
    I'm a C object
    """
    print(c.__class__.__name__)
    print(c.__class__())

@cython.locals(d=object)
def test_object(o):
    """
    >>> test_object({})
    dict
    {}
    >>> test_object(1)
    int
    0
    >>> test_object(C())
    C
    I'm a C object
    """
    print(o.__class__.__name__)
    print(o.__class__())



# ticket: 6651
# mode: run
# tag: cpp, cpp17

cimport cython

from libcpp.string cimport string
from libcpp.string_view cimport string_view, npos
from cython.operator cimport dereference as deref, preincrement as preinc, predecrement as predec

b_asdf = b'asdf'
b_asdg = b'asdg'
b_s = b's'


cdef int compare_to_asdf(string_view s) except -999:
    return s.compare(b"asdf")

def test_coerced_literal_ref():
    """
    >>> test_coerced_literal_ref()
    0
    """
    return compare_to_asdf("asdf")


cdef return_to_py():
    return string_view(b"asdf")

def test_return_to_py():
    """
    >>> test_return_to_py() == b_asdf
    True
    """
    return return_to_py()


def test_constructors(py_str):
    """
    >>> b_s_7 = b_s * 7
    >>> test_constructors(b_s_7) == (b'', b_s, b_s * 3) + (b_s_7, ) * 5
    True
    """
    cdef string s = py_str
    cdef string_view view1 = s
    cdef string_view substr1 = view1.substr(3, 1)
    cdef string_view substr2 = string_view(view1.data(), 3)
    cdef string_view view2 = string_view(view1.data())
    cdef string_view view3 = string_view(view1)
    cdef string_view view4 = string_view(view1.data(), view1.size())
    cdef string_view view5
    view5 = view2

    return (string_view(), substr1, substr2, view1, view2, view3, view4, view5)


def test_iterators(string_view s):
    """
    >>> test_iterators(b'XXX hello, world! XXX')
    (True, True)
    """
    cdef size_t c1 = 0, c2 = 0

    cdef string_view.iterator it = s.begin()
    cdef string_view.reverse_iterator rit = s.rbegin()

    preinc(it)
    preinc(rit)

    assert deref(it + 1) == deref(rit + 1)
    assert deref(it - 1) == deref(rit - 1)

    assert it - 1 < it and it < it + 1
    assert it - 1 <= it and it <= it + 1
    assert it + 1 > it and it > it - 1
    assert it + 1 >= it and it >= it - 1

    assert rit - 1 < rit and rit < rit + 1
    assert rit - 1 <= rit and rit <= rit + 1
    assert rit + 1 > rit and rit > rit - 1
    assert rit + 1 >= rit and rit >= rit - 1

    assert it == it and it != it + 1
    assert rit == rit and rit != rit + 1

    predec(it)
    predec(rit)

    while it != s.end():
        preinc(it)
        preinc(c1)

    while rit != s.rend():
        preinc(rit)
        preinc(c2)

    return c1 == s.size(), c1 == c2


def test_access(string_view s, i):
    """
    >>> test_access(b_asdf, 1) == (ord(b_asdf[1:2]), ) * 2
    True
    >>> test_access(b_asdf, 5)  # doctest: +ELLIPSIS
    Traceback (most recent call last):
        ...
    IndexError: ...
    """
    return s.at(i), s[i]


def test_empty(string_view s1, string_view s2):
    """
    >>> test_empty(b'', b_asdf)
    (True, False)
    """
    return s1.empty(), s2.empty()


def test_size(string_view s):
    """
    >>> test_size(b_asdf)
    (4, 4)
    """
    return s.size(), s.length()


def test_npos(string_view a, string_view b):
    """
    >>> test_npos(b'abc', b'x')
    True
    >>> test_npos(b'abc', b'a')
    False
    """
    return a.find(b) == npos


def test_compare(string_view s1, string_view s2):
    """
    >>> all(map(lambda x: x == 0, test_compare(b_asdf, b_asdf)))
    True

    >>> all(map(lambda x: x > 0, test_compare(b_asdf, b_asdg)))
    True
    """
    return (s2.compare(s1),
            s2.compare(0, s1.size(), s1),
            s2.compare(0, s1.size(), s1, 0, s1.size()),
            s2.compare(s1.data()),
            s2.compare(0, s1.size(), s1.data()),
            s2.compare(0, s1.size(), s1.data(), s1.size()))


def test_substr(string_view s):
    """
    >>> b_test_str = b'ABCDEFGH'
    >>> test_substr(b_test_str) == (b_test_str, b_test_str[1:], b_test_str[1:5])
    True
    """
    cdef string_view o1, o2, o3
    o1 = s.substr()
    o2 = s.substr(1)
    o3 = s.substr(1, 4)
    return o1, o2, o3


def test_copy(string_view s):
    """
    >>> test_copy(b_asdf) == (b_asdf[:3], b_asdf[1:])
    True
    """
    cdef char[5] buffer1, buffer2
    cdef size_t length1 = s.copy(buffer1, 3)
    cdef size_t length2 = s.copy(buffer2, 4, 1)
    buffer1[length1] = c'\0'
    buffer2[length2] = c'\0'
    return buffer1, buffer2


def test_swap(string_view s1, string_view s2):
    """
    >>> test_swap(b_asdf, b_asdg) == (b_asdg, b_asdf)
    True
    """
    s1.swap(s2)
    return s1, s2


def test_find(string_view s1, string_view s2):
    """
    >>> all(map(lambda x: x == 2, test_find(b_asdf, b_asdf[2:])))
    True
    """
    return (s1.find(s2),
            s1.find(s2, 1),
            s1.find(s2.data(), 0, s2.size()),
            s1.find(s2.data()),
            s1.find(s2.data(), 0),
            s1.find(s2.at(0), 0))


def test_rfind(string_view s1, string_view s2):
    """
    >>> all(map(lambda x: x == 2, test_rfind(b_asdf, b_asdf[2:])))
    True
    """
    return (s1.rfind(s2),
            s1.rfind(s2, s1.size() - 2),
            s1.rfind(s2.data(), s1.size() - 2, s2.size()),
            s1.rfind(s2.data()),
            s1.rfind(s2.data(), s1.size() - 2),
            s1.rfind(s2.at(0), s1.size() - 2))


@cython.test_assert_path_exists("//PythonCapiCallNode")
@cython.test_fail_if_path_exists("//AttributeNode")
def test_decode(string_view a):
    """
    >>> print(test_decode(b_asdf))
    asdf
    """
    return a.decode('ascii')


@cython.test_assert_path_exists("//PythonCapiCallNode")
@cython.test_fail_if_path_exists("//AttributeNode")
def test_decode_sliced(string_view a):
    """
    >>> print(test_decode_sliced(b_asdf))
    sd
    """
    return a[1:3].decode('ascii')


def test_equals_operator(char *a, string_view b):
    """
    >>> test_equals_operator(b_asdf, b_asdf)
    (True, False)
    """
    cdef string_view s = string_view(a)
    return b == s, b != <char *>"asdf"


def test_less_than(string_view a, char *b):
    """
    >>> test_less_than(b_asdf[:-1], b_asdf)
    (True, True, True)

    >>> test_less_than(b_asdf[:-1], b_asdf[:-1])
    (False, False, True)
    """
    cdef string_view t = string_view(b)
    return (a < t, a < b, a <= b)


def test_greater_than(string_view a, char *b):
    """
    >>> test_greater_than(b_asdf[:-1], b_asdf)
    (False, False, False)

    >>> test_greater_than(b_asdf[:-1], b_asdf[:-1])
    (False, False, True)
    """
    cdef string_view t = string_view(b)
    return (a > t, a > b, a >= b)


def test_iteration(string_view s):
    """
    >>> test_iteration(b'xyz')
    [120, 121, 122]
    >>> test_iteration(b'')
    []
    """
    return [c for c in s]
